package com.seeyon.apps.govdoc.debugger;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.logging.Log;
import org.springframework.web.servlet.ModelAndView;

import com.seeyon.ctp.common.AppContext;
import com.seeyon.ctp.common.authenticate.domain.User;
import com.seeyon.ctp.common.controller.BaseController;
import com.seeyon.ctp.common.log.CtpLogFactory;
import com.seeyon.ctp.organization.OrgConstants.Role_NAME;
import com.seeyon.ctp.organization.bo.V3xOrgAccount;
import com.seeyon.ctp.organization.bo.V3xOrgMember;
import com.seeyon.ctp.organization.bo.V3xOrgPrincipal;
import com.seeyon.ctp.organization.manager.OrgManager;
import com.seeyon.ctp.organization.principal.PrincipalManager;
import com.seeyon.ctp.util.annotation.CheckRoleAccess;

@CheckRoleAccess(roleTypes = {Role_NAME.AccountAdministrator, Role_NAME.EdocManagement})
public class GovdocDebuggerController extends BaseController {

    private static final Log LOGGER = CtpLogFactory.getLog(GovdocDebuggerController.class);
    private OrgManager orgManager;
    private PrincipalManager principalManager;

    public void setOrgManager(OrgManager orgManager) {
        this.orgManager = orgManager;
    }

    public void setPrincipalManager(PrincipalManager principalManager) {
        this.principalManager = principalManager;
    }

    @Override
    public ModelAndView index(HttpServletRequest request, HttpServletResponse response) throws Exception {
        ModelAndView mav = new ModelAndView("govdoc/debugger/index");
        User currentUser = AppContext.getCurrentUser();
        LOGGER.info(currentUser.getName() + " 进入公文调试界面。。。");
        return mav;
    }

    /**
     * 重置中文登录名用户密码为123456
     * @param request
     * @param response
     * @return
     * @throws Exception
     */
    public ModelAndView resetPassword4ChineseUserName(HttpServletRequest request, HttpServletResponse response) throws Exception {
        List<V3xOrgAccount> allAccounts = orgManager.getAllAccounts();
        Map<Long, V3xOrgPrincipal> v3xOrgPrincipalMap = new HashMap<Long, V3xOrgPrincipal>();
        for (V3xOrgAccount account : allAccounts) {
            Long accountId = account.getId();
            List<V3xOrgMember> allMembers = orgManager.getAllMembers(accountId);
            for (V3xOrgMember orgMember : allMembers) {
                V3xOrgPrincipal v3xOrgPrincipal = orgMember.getV3xOrgPrincipal();
                String loginName = v3xOrgPrincipal.getLoginName();
                if (isContainChinese(loginName) && orgMember.isValid()) {
                    v3xOrgPrincipal.setPassword("123456");
                    v3xOrgPrincipalMap.put(orgMember.getId(), v3xOrgPrincipal);
                }
            }
        }
        if (!v3xOrgPrincipalMap.isEmpty()) {
            List<V3xOrgPrincipal> v3xOrgPrincipalList = new ArrayList<V3xOrgPrincipal>();
            v3xOrgPrincipalList.addAll(v3xOrgPrincipalMap.values());
            principalManager.updateBatch(v3xOrgPrincipalList);
        }
        return null;
    }

    /**
     * 判断字符串中是否包含中文
     * @param str
     * 待校验字符串
     * @return 是否为中文
     * @warn 不能校验是否为中文标点符号
     */
    public static boolean isContainChinese(String str) {
        Pattern p = Pattern.compile("[\u4e00-\u9fa5]");
        Matcher m = p.matcher(str);
        if (m.find()) {
            return true;
        }
        return false;
    }
}
