package com.seeyon.apps.trustdo.dao.impl;

import java.sql.SQLException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import com.seeyon.ctp.common.exceptions.BusinessException;
import com.seeyon.ctp.util.JDBCAgent;
import org.apache.commons.logging.Log;

import com.seeyon.apps.trustdo.dao.XRDDao;
import com.seeyon.apps.trustdo.po.XRDUserPO;
import com.seeyon.ctp.common.dao.BaseHibernateDao;
import com.seeyon.ctp.common.log.CtpLogFactory;
import com.seeyon.ctp.util.DBAgent;
import com.seeyon.ctp.util.FlipInfo;
import com.seeyon.ctp.util.Strings;

@SuppressWarnings("unchecked")
public class XRDDaoImpl extends BaseHibernateDao<XRDUserPO> implements XRDDao {

    private static final Log LOGGER = CtpLogFactory.getLog(XRDDaoImpl.class);

    @Override
    public void save(XRDUserPO xrdPo) {
        DBAgent.saveOrUpdate(xrdPo);
    }

    @Override
    public void del(XRDUserPO xrdPo) {
        DBAgent.delete(xrdPo);
    }

    @Override
    public XRDUserPO getOneByid(Long id) {
        XRDUserPO xo = DBAgent.get(XRDUserPO.class, id);
        return xo;
    }

    @Override
    public List<XRDUserPO> getAll(Map<String, Object> conditions, FlipInfo fi) {
        String seeyonLoginNameCondition = (String) conditions.get("seeyonLoginName");
        String seeyonUserDeptCondition = (String) conditions.get("seeyonUserDept");
        String seeyonUserNameCondition = (String) conditions.get("seeyonUserName");
        List<XRDUserPO> list = new ArrayList<XRDUserPO>();
        StringBuilder hql = new StringBuilder("SELECT " +
                "xrd.id, " +
                "mem.NAME as seeyonUserName," +
                "pri.LOGIN_NAME as seeyonLoginName, " +
                "  un.NAME as seeyonUserDept, " +
                "  acc.NAME as extend2," +
                "xrd.bind_time as bindTime " +
                "FROM " +
                "trustdo_bind_user xrd " +
                "INNER JOIN org_member AS mem ON xrd.member_id = mem.id " +
                "INNER JOIN org_principal pri ON mem.id = pri.MEMBER_ID " +
                "INNER JOIN org_unit un ON mem.ORG_DEPARTMENT_ID = un.id " +
                "INNER JOIN org_unit acc ON mem.ORG_ACCOUNT_ID = acc.id " +
                "where 1=1 ");
        if (Strings.isNotBlank(seeyonLoginNameCondition)) {
            hql.append(" and pri.login_name like '%" + seeyonLoginNameCondition + "%' ");
        }
        if (Strings.isNotBlank(seeyonUserDeptCondition)) {
            hql.append(" and un.name like '%" + seeyonUserDeptCondition + "%' ");
        }
        if (Strings.isNotBlank(seeyonUserNameCondition)) {
            hql.append(" and mem.name like '%" + seeyonUserNameCondition + "%' ");
        }
        hql.append(" ORDER BY xrd.bind_time desc ");
        JDBCAgent jdbc = new JDBCAgent(true);
        try {
            try {
                FlipInfo byPaging = jdbc.findByPaging(hql.toString(), fi);
                list = byPaging.getData();
            } catch (BusinessException e) {
                e.printStackTrace();
            }
        } catch (SQLException e) {
            logger.error(e);
        } finally {
            jdbc.close();
        }
        return list;
    }

    @Override
    public List<XRDUserPO> getOneByTrustdoAccount(Long account) {
        Map<String, Object> param = new HashMap<String, Object>();
        StringBuilder hql = new StringBuilder("from XRDUserPO as xup WHERE 1=1");
        hql.append(" AND xup.memberId=:account");
        param.put("account", account);
        LOGGER.debug(hql.toString());
        List<XRDUserPO> result = DBAgent.find(hql.toString(), param);
        return result;
    }
}
