/**
 * $Author$
 * $Rev$
 * $Date::                     $:
 *
 * Copyright (C) 2012 Seeyon, Inc. All rights reserved.
 *
 * This software is the proprietary information of Seeyon, Inc.
 * Use is subject to license terms.
 */

package com.seeyon.apps.weboffice.controller;

import java.io.BufferedOutputStream;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.PrintWriter;
import java.net.URLDecoder;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.Date;
import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.io.FileUtils;
import org.apache.commons.io.IOUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.seeyon.ctp.common.AppContext;
import com.seeyon.ctp.common.authenticate.domain.User;
import com.seeyon.ctp.common.encrypt.CoderFactory;
import com.seeyon.ctp.common.exceptions.BusinessException;
import com.seeyon.ctp.common.filemanager.Constants;
import com.seeyon.ctp.common.filemanager.manager.AttachmentManager;
import com.seeyon.ctp.common.filemanager.manager.FileManager;
import com.seeyon.ctp.common.office.manager.OfficeBakFileManager;
import com.seeyon.ctp.common.office.po.OfficeBakFile;
import com.seeyon.ctp.common.po.filemanager.Attachment;
import com.seeyon.ctp.common.po.filemanager.V3XFile;
import com.seeyon.ctp.util.Datetimes;
import com.seeyon.ctp.util.Strings;
import com.seeyon.ctp.util.UUIDLong;
import com.seeyon.ctp.util.json.JSONUtil;

import DBstep.iMsgServer2000;

/**
 * Wps及Ofd正文保存及查看接口
 *
 */
public class WPSOfficeServlet extends HttpServlet {
    /**
     *
     */
    private static final long serialVersionUID = 7681686112649227064L;

    private static Log LOGGER = LogFactory.getLog(WPSOfficeServlet.class);

    private FileManager fileManager = (FileManager)AppContext.getBeanWithoutCache("fileManager");
    private AttachmentManager   attachmentManager = (AttachmentManager)AppContext.getBean("attachmentManager");
	private OfficeBakFileManager officeBakFileManager = (OfficeBakFileManager)AppContext.getBean("officeBakFileManager");

    @ Override
    protected void doGet(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
        doPost(request, response);
    }

    @Override
    protected void doPost(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
        String flag = request.getParameter("flag");
        if (Strings.isBlank(flag)) {
        	LOGGER.error("未给定操作类型flag的值（flag取值[save:保存,open:打开）");
            return;
        }
        //保存文件
        if (flag.toLowerCase().contains("save")) {
            saveFile(request, response);
        }
        //打开文件
        else if (flag.toLowerCase().contains("open")) {
            openFile(request, response);
        }
        //打开模板文件
        else if (flag.toLowerCase().contains("template")) {
            openTemplateFile(request, response);
        }
        //获取附件
        else if (flag.toLowerCase().contains("attachement")) {
            findAttachement(request, response);
        }
        //copy文件
        else if (flag.toLowerCase().contains("copy")) {
        	copyFile(request, response);
        	//下载文件，附件套红时用到
        }else if(flag.toLowerCase().contains("download")){
        	downaloadFile(request, response);
        }
    }

	private void downaloadFile(HttpServletRequest request, HttpServletResponse response) {
		String fileId=request.getParameter("fileId");
		try {
			V3XFile v3xFile = fileManager.getV3XFile(Long.valueOf(fileId));
			if(null!=v3xFile){
				String filePath=this.fileManager.getFolder(v3xFile.getCreateDate(), true)+ File.separator + fileId;
				File file=new File(filePath);
				if(file.exists()){
					outputFile(response, file);
				}
			}
		} catch (NumberFormatException e) {
			e.printStackTrace();
		} catch (BusinessException e) {
			e.printStackTrace();
		}
	}

    /**
     *
     * @param request
     * @param response
     * @return
     */
    public String openTemplateFile(HttpServletRequest request, HttpServletResponse response){

        String rs = null;
        try {
            String reqStr = request.getParameter("filepath");
            if (Strings.isBlank(reqStr)) {
                return null;
            }
            reqStr = URLDecoder.decode(reqStr, "UTF-8");
            String templatePath =java.net.URLDecoder.decode(reqStr,"UTF-8"); ;// 公文标识
            File srcFile = new File(templatePath);
            outputFile(response, srcFile);
        } catch(Exception e) {
        	LOGGER.error("打开文件模板出错", e);
        }
        return rs;
    }


    /**
     * 获取web容器临时文件夹
     * @param request
     * @return
     */
    private String WebTempFolder(HttpServletRequest request) {
        File dest = new File(request.getSession().getServletContext().getRealPath("/")+File.separator+"temp");
        if (!dest.exists()){
        	dest.mkdirs();
        }
        return dest.getAbsolutePath();
    }
    /**
     * 处理客户面打开文件的请求，只支持wps和ofd文件的打开
     * @param request
     * @param response
     * @return
     */
    private String openFile(HttpServletRequest request, HttpServletResponse response) {
        String rs = null;
        try {
        	//正文ID
        	String fileId = request.getParameter("fileId");;
        	if (Strings.isBlank(fileId)){
        		LOGGER.error("文档标识为空，无法获取文件！");
        		LOGGER.error("根据标识【"+fileId+"】未找到记录，无法获取文档！");
        		return null;
        	}
        	File file = fileManager.getFile(Long.valueOf(fileId));
        	if(file == null) {
        		LOGGER.error("服务器上文件为空：" + fileId);
        		return null;
        	}
        	//将金格的正文进行处理
        	iMsgServer2000 msgObj = new iMsgServer2000();
        	byte2File(msgObj.ToDocument(file2byte(file)), file);;
        	outputFile(response, file);
        } catch(Exception e) {
        	LOGGER.error("打开文档出错", e);
        }
        return rs;
    }

    private void outputFile(HttpServletResponse response, File file) {
    	if (file.exists()) {
    		InputStream in = null;
    		OutputStream out = null;
    		try {
				in = new FileInputStream(file);
				response.setContentType("application/x-msdownload; charset=UTF-8");
				response.setHeader("Content-disposition", "attachment;");
				response.setContentLength(in.available());
				out = response.getOutputStream();
				CoderFactory.getInstance().download(in, out);
			} catch (Exception e) {
				e.printStackTrace();
			}finally{
	            IOUtils.closeQuietly(in);
	            IOUtils.closeQuietly(out);
			}
		}
	}

	/**
     * linux环境下保存文件包括wps/ofd 根据format 值判断是哪一种
     * @param request
     * @param response
     * @return
     */
    public String saveFile(HttpServletRequest request, HttpServletResponse response) {
        String rs = "sucess"; // 操作成功的返回值，错误时，返回错误信息
        long contentSize = request.getContentLength();
        //String summaryId = request.getParameter("summaryId");
        // 获取文件路径
        String fileId = request.getParameter("fileId");
        if (Strings.isBlank(fileId)) {
        	LOGGER.error("文档标识为空，无法获取文件！");
    		return "未指定文档标识，无法保存文档";
    	}
        /* String fileType = request.getParameter("format");

        String ofdFileId = request.getParameter("ofdFileId");

        if ("ofd".equals(fileType))
        {
            contenId = ofdFileId;
        }
        else if ("wps".equals(fileType))
        {
            contenId = fileId;
        }
        if (Strings.isEmpty(contenId))
        {
            contenId = String.valueOf(UUIDLong.longUUID()); //为空时生成一个新的
        }*/
        if (contentSize > 0 && Strings.isNotBlank(fileId)) {
            boolean isSuccess = saveFileToDisk(request, fileId);
            if (!isSuccess) {
                rs = "物理文件保存失败！";
            }
        }
        return rs;
    }
     /**
      * 保存文件到磁盘中
      * @param request 客户端请求对象
      * @param contenId 文档的文件标识，将用作文件名
      * @return 保存是否成功
      */
    public boolean saveFileToDisk(HttpServletRequest request, String contenId) {
    	LOGGER.info(this.getClass().getSimpleName()+" 保存文件contenId: ["+contenId+"] 操作系统："+   request.getParameter("os"));
        boolean rs = false;
        try {
            InputStream in = request.getInputStream();
            if (request.getContentLength() > 0) {
                String os = request.getParameter("os");
                String name = contenId;
                if (Strings.isNotBlank(os)) {
                	//wps在window下提交的数据前128字节是用来保存一些文件名相关信息的
                    if (os.toLowerCase().contains("win")) {
                        // 固定128字节
                        byte filename[] = new byte[128];
                        int filenamelen;

                        if ((filenamelen = in.read(filename)) != -1) {
                        	for(int i=0;i<filename.length;i++){
                        		if(filename[i]==0){
                        			filenamelen=i;
                        			break;
                        		}
                        	}
                        	name = new String(filename, 0, filenamelen);
                        }
                    } else {
                        name = request.getHeader("filename");
                    }
                    if (name == null || "".equals(name)) {//未指定名称时，默认与标识相同
                        name = contenId;
                    }
                }
                Date createDate = new Date();
                V3XFile v3xFile = fileManager.getV3XFile(Long.valueOf(contenId));
                boolean isNew = false;
                if (v3xFile == null) {
                    v3xFile = new V3XFile(Long.valueOf(contenId));
                    v3xFile.setCreateDate(createDate);
                    isNew = true;
                } else {
                    createDate = v3xFile.getCreateDate();
                    name=v3xFile.getFilename();
                }
                File dest = new File(this.fileManager.getFolder(createDate, true) + File.separator + contenId);
                FileOutputStream out = new FileOutputStream(dest);
                int count = IOUtils.copy(in, out);
                //FileEncryptAndDecrypt.encryptFile(dest);//加密文档
                v3xFile.setSize(Long.valueOf(count));
                v3xFile.setFilename(name);
                v3xFile.setId(Long.valueOf(contenId));
                v3xFile.setUpdateDate(new Date());
                // wpsFile.setType();//???
                v3xFile.setCategory(1);
                String fileType = request.getParameter("format");
                if (fileType != null) {
                    if (fileType.toLowerCase().contains("wps")) {
                        v3xFile.setMimeType("application/wps");
                    } else if (fileType.toLowerCase().contains("ofd")) {
                        v3xFile.setMimeType("application/ofd");
                    }
                }
                String noMillisecondTime = Datetimes.format(new Date(System.currentTimeMillis()),"yyyy-MM-dd HH:mm:ss");
                v3xFile.setUpdateDate(Datetimes.parseDate(noMillisecondTime));
                User user = AppContext.getCurrentUser();
                if (user != null) {
                    v3xFile.setCreateMember(user.getId());
                    v3xFile.setAccountId(user.getAccountId());
                }
                if (!isNew) {
                    fileManager.update(v3xFile);
                    Attachment attachment = attachmentManager.getAttachmentByFileURL(v3xFile.getId());
                    if(attachment != null) {
	                    attachment.setCreatedate(new Date());
	                    attachmentManager.update(attachment);
                    }
                } else {
                    fileManager.save(v3xFile);
                }
                in.close();
                rs = true;
            }
        } catch(Exception e) {
        	rs = false;
        	LOGGER.error("保存文件到磁盘报错", e);
        }
        return rs;
    }


    /**
     * 因流式和版式打开的远程文件的方法，是完整的url链接地址，所以会把保存在系统指定目录下（如base）的文件复制一份到应用服务器的指定目录下，
     * 再将此文件的链接地址返给客户端
     * @param request 请求对象
     * @param fileId 文件标识
     * @param format 文件格式，为子文件夹路径
     * @return
     * @throws BusinessException
     * @throws IOException
     */
    private String getRelativePath(HttpServletRequest request, String fileId, String format) throws BusinessException, IOException {
        long contentId = Long.valueOf(fileId);
        // String format =request.getParameter("format");
        V3XFile vf = fileManager.getV3XFile(Long.valueOf(contentId));
        if (vf == null) {
        	LOGGER.info("未找到指定id为 【" + contentId + "】的文件，或者文件已不存在！");
            return "";
        }
        File dfile = fileManager.getFile(contentId);
        if (dfile == null) {
            return "";
        }
        String fileType = request.getParameter("format");
        String destName = vf.getFilename();
        Pattern p = Pattern.compile("[\u4e00-\u9fa5]");
        Matcher m = p.matcher(destName);
        if (m.find()) {
            //当文档名称含有中文时，重新定义
        	destName=String.valueOf(UUIDLong.longUUID());
        }
        if ("ofd".equals(fileType)){
            destName += ".ofd";
        }

        Date d = vf.getCreateDate();
        String rs = fileManager.getFolder(d, true) + "/" + destName;

        File dest = new File(fileManager.getFolder(d, true) + "/" + destName);
        FileUtils.copyFile(fileManager.getFile(contentId),dest);
//        FileEncryptAndDecrypt.decryptFile(dest);
        return rs;
    }

    /**
     * 查找
     * @param request
     * @param response
     */
    private String findAttachement(HttpServletRequest request, HttpServletResponse response) {
    	String summaryId = request.getParameter("summaryId");
    	if (Strings.isBlank(summaryId) || "undefined".equals(summaryId)) {
    		LOGGER.error("文档标识为空，无法获取文件！");
    		return null;
    	}
    	//只查找正文的附件。
    	 try {
    		 String attListJSON = "";
    		 List<Attachment> attsList = attachmentManager.getByReference(Long.parseLong(summaryId));
    		 if(attsList != null && !attsList.isEmpty()){
    			 attListJSON = JSONUtil.toJSONString(attachmentManager.setOfficeTransformEnable(fileNameESCHtml(request,attsList)));
    		 }
			 response.setCharacterEncoding("UTF-8");
			 response.setContentType("application/json; charset=utf-8");
			 PrintWriter out = response.getWriter();
			 if (out != null) {
				 out = response.getWriter();
				 out.append(attListJSON);
				 out.close();
			 }
		} catch (Exception e) {
			LOGGER.error("获取附件信息失败！", e);
		}
        return null;
    }

    private  List<Attachment> fileNameESCHtml(HttpServletRequest request,List<Attachment>  list){
    	String format = request.getParameter("format");// 文件格式(wps,ofd)
        List<Attachment> attlist=new ArrayList<Attachment>();
        for (Attachment att : list) {
        	//文单的附件,过滤掉意见区附件、文单关联附件
        	if(att.getCategory() == 4 &&att.getReference().longValue() == att.getSubReference().longValue()&& (att.getExtension().indexOf("doc") > -1 || att.getExtension().indexOf("docx") > -1 || att.getExtension().indexOf("wps") > -1)&&att.getType()==Constants.ATTACHMENT_TYPE.FILE.ordinal()){
        		Attachment attc;
        		try {
        			attc = (Attachment)att.clone();
        			attc.setId(att.getId());
        			attc.setFilename(Strings.toHTML(attc.getFilename().replace((char)160, (char)32)).replaceAll("&quot;", "\\\""));
//        			attc.setDescription(getAttachmentPath(request, attc.getFileUrl(), format));
        			attc.setDescription(attc.getFileUrl().toString());
        			attlist.add(attc);
        		} catch (Exception e) {
        			LOGGER.error("",e);
        		}
        	}
        }
        return attlist;
    }
    /**
     * 文件附件路径获取
     * @param request
     * @param fileId
     * @param format
     * @return
     * @throws BusinessException
     * @throws IOException
     */
    private String getAttachmentPath(HttpServletRequest request, Long fileId, String format) throws BusinessException, IOException {
        V3XFile vf = fileManager.getV3XFile(fileId);
        if (vf == null) {
        	LOGGER.info("未找到指定id为 【" + fileId + "】的文件，或者文件已不存在！");
            return "";
        }
        String destName = String.valueOf(vf.getId());
        Date d = vf.getCreateDate();
        String rs = fileManager.getFolder(d, true) + "/" + destName;
        
        return rs;
    }
    /**
     * 保存文件到磁盘中
     * @param request 客户端请求对象
     * @param contenId 文档的文件标识，将用作文件名
     * @return 保存是否成功
     */
    public String copyFile(HttpServletRequest request, HttpServletResponse response) {
    	LOGGER.info("保存文件开始");
    	String rs = "sucess"; // 操作成功的返回值，错误时，返回错误信息
    	boolean rss = false;
    	Date today = new Date();
    	long contentSize = request.getContentLength();
    	//String summaryId = request.getParameter("summaryId");
    	// 获取文件路径
    	String fileId = request.getParameter("fileId");
    	if (Strings.isBlank(fileId)) {
    		LOGGER.error("文档标识为空，无法获取文件！");
    		return "未指定文档标识，无法保存文档";
    	}
    	if (contentSize > 0 && Strings.isNotBlank(fileId)) {
    		try {
    			InputStream in = request.getInputStream();
    			if (request.getContentLength() > 0) {
    				String os = request.getParameter("os");
    				String name = fileId;
    				if (Strings.isNotBlank(os)) {
    					//wps在window下提交的数据前128字节是用来保存一些文件名相关信息的
    					if (os.toLowerCase().contains("win")) {
    						// 固定128字节
    						byte filename[] = new byte[128];
    						int filenamelen;

    						if ((filenamelen = in.read(filename)) != -1) {
                            	for(int i=0;i<filename.length;i++){
                            		if(filename[i]==0){
                            			filenamelen=i;
                            			break;
                            		}
                            	}
    							name = new String(filename, 0, filenamelen);
    						}
    					} else {
    						name = request.getHeader("filename");
    					}
    					if (name == null || "".equals(name)) {//未指定名称时，默认与标识相同
    						name = fileId;
    					}
    				}
    				//这里是正文文件
    				V3XFile v3xFile = fileManager.getV3XFile(Long.valueOf(fileId));
    				File file = fileManager.getFile(v3xFile.getId());
    				String contentFilePath = file.getPath();
    				
    				//备份物理文件，防止文件丢失。
    				List<Long> obfList = officeBakFileManager.getOfficeBakFileIds(Long.valueOf(fileId));
    				List<V3XFile> fs = new ArrayList<V3XFile>();
    				Long[] fileIds = null;
    				if (obfList.size() > 0) {
    					fileIds = new Long[obfList.size()];
    					for(int i =0;i<obfList.size();i++){
    						fileIds[i] = obfList.get(i);
    					}
    					fs = fileManager.getV3XFile(fileIds);
    	 			}
    				//如果保存的花脸多余5个，根据时间删除最早的，只留下4个
    				Collections.sort(fs,  new Comparator<V3XFile>(){
    					@Override
    					public int compare(V3XFile o1, V3XFile o2) {
    						if(o1.getCreateDate().after(o2.getCreateDate())){
    							return 1;
    						}
    						return -1;
    					}
    				});
    				V3XFile tempFile=null;
    				while(fs!=null && fs.size()>=5){
    					tempFile=fs.remove(0);
    					fileManager.deleteFile(tempFile.getId(),true);
    					officeBakFileManager.delete(tempFile.getId());
    				}
    				V3XFile bakV3Xfile = (V3XFile) v3xFile.clone();
    				bakV3Xfile.setFilename("copy_" + fileId);
    				bakV3Xfile.setId(UUIDLong.longUUID());
    				bakV3Xfile.setFilename("copy"+fileId.toString());
    				bakV3Xfile.setCreateDate(today);
    				bakV3Xfile.setUpdateDate(today);
    				String fileType = request.getParameter("format");
    				if (fileType != null) {
    					if (fileType.toLowerCase().contains("wps")) {
    						bakV3Xfile.setMimeType("application/wps");
    					} else if (fileType.toLowerCase().contains("ofd")) {
    						bakV3Xfile.setMimeType("application/ofd");
    					}
    				}
    				User user = AppContext.getCurrentUser();
    				if (user != null) {
    					bakV3Xfile.setCreateMember(user.getId());
    					bakV3Xfile.setAccountId(user.getAccountId());
    				}
    				fileManager.save(bakV3Xfile);
    				String hualianFilePath = fileManager.getFolder(today, true) + File.separator + bakV3Xfile.getId();
    				
    				OfficeBakFile officeBakFile = new OfficeBakFile();
    				officeBakFile.setFileId(bakV3Xfile.getId());
    				officeBakFile.setSourceId(Long.valueOf(fileId));
    				officeBakFileManager.save(officeBakFile);//保存表间关系
    				
					FileUtils.copyFile(file, new File(hualianFilePath));

    				in.close();
    				rss = true;
    			}
    		} catch(Exception e) {
    			rss = false;
    			LOGGER.error("保存文件到磁盘报错", e);
    		}
    	}
    	if (!rss) {
    		rs = "物理文件保存失败！";
    	}
    	LOGGER.info("保存文件结束");
    	return rs;
    }
	
	private byte[] file2byte(File file) {
		byte[] buffer = null;
		FileInputStream fis = null;
		ByteArrayOutputStream bos = null;
		try {
			fis = new FileInputStream(file);
			bos = new ByteArrayOutputStream();
			byte[] b = new byte[1024];
			int n;
			while ((n = fis.read(b)) != -1) {
				bos.write(b, 0, n);
			}

			buffer = bos.toByteArray();
		} catch (FileNotFoundException e) {
			LOGGER.error("文件没有找到",e);
		} catch (IOException e) {
			LOGGER.error("文件Io异常",e);
		}finally {
			if(bos != null){
				try {
					bos.close();
				}catch (IOException e){
					LOGGER.error("关闭输出流异常",e);
				}
			}
			if(fis != null){
				try {
					fis.close();
				}catch (IOException e){
					LOGGER.error("关闭输入流异常",e);
				}
			}
		}
		return buffer;
	}
	
	public static void byte2File(byte[] buf, File file) {
		BufferedOutputStream bos = null;
		FileOutputStream fos = null;
		try {
			fos = new FileOutputStream(file);
			bos = new BufferedOutputStream(fos);
			bos.write(buf);
		} catch (Exception e) {
			LOGGER.error("文件输出异常",e);
		} finally {
			if (bos != null) {
				try {
					bos.close();
				} catch (IOException e) {
					LOGGER.error("关闭输出流异常",e);
				}
			}
			if (fos != null) {
				try {
					fos.close();
				} catch (IOException e) {
					LOGGER.error("关闭输入流异常",e);
				}
			}
		}
	}
}
