package com.seeyon.v3x.edoc.manager;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.UUID;
import java.util.concurrent.ConcurrentHashMap;

import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang.StringUtils;
import org.apache.commons.logging.Log;

import com.seeyon.apps.edoc.constants.EdocConstant;
import com.seeyon.apps.govdoc.exchange.util.GovDocUtil;
import com.seeyon.ctp.common.AppContext;
import com.seeyon.ctp.common.ModuleType;
import com.seeyon.ctp.common.SystemEnvironment;
import com.seeyon.ctp.common.affair.manager.AffairManager;
import com.seeyon.ctp.common.config.manager.ConfigManager;
import com.seeyon.ctp.common.constants.ProductEditionEnum;
import com.seeyon.ctp.common.constants.SystemProperties;
import com.seeyon.ctp.common.ctpenumnew.EnumNameEnum;
import com.seeyon.ctp.common.exceptions.BusinessException;
import com.seeyon.ctp.common.filemanager.dao.V3XFileDAO;
import com.seeyon.ctp.common.filemanager.manager.FileManager;
import com.seeyon.ctp.common.filemanager.manager.PartitionManager;
import com.seeyon.ctp.common.flag.SysFlag;
import com.seeyon.ctp.common.i18n.ResourceUtil;
import com.seeyon.ctp.common.log.CtpLogFactory;
import com.seeyon.ctp.common.permission.vo.PermissionVO;
import com.seeyon.ctp.common.po.affair.CtpAffair;
import com.seeyon.ctp.common.po.config.ConfigItem;
import com.seeyon.ctp.common.po.filemanager.V3XFile;
import com.seeyon.ctp.common.po.template.CtpTemplateCategory;
import com.seeyon.ctp.form.api.FormApi4Cap3;
import com.seeyon.ctp.form.bean.FormAuthViewBean;
import com.seeyon.ctp.form.bean.FormAuthViewFieldBean;
import com.seeyon.ctp.form.bean.FormBean;
import com.seeyon.ctp.form.bean.FormFieldBean;
import com.seeyon.ctp.form.bean.FormFieldComBean.FormFieldComEnum;
import com.seeyon.ctp.form.bean.FormViewBean;
import com.seeyon.ctp.form.biz.exception.BizException;
import com.seeyon.ctp.form.biz.vo.BizValidateResultVO;
import com.seeyon.ctp.form.manager.GovdocFormExtendManager;
import com.seeyon.ctp.form.manager.GovdocFormOpinionSortManager;
import com.seeyon.ctp.form.po.FormOwner;
import com.seeyon.ctp.form.po.FormPermissionConfig;
import com.seeyon.ctp.form.po.FormResource;
import com.seeyon.ctp.form.po.GovdocFormExtend;
import com.seeyon.ctp.form.po.GovdocFormOpinionSort;
import com.seeyon.ctp.form.util.Enums;
import com.seeyon.ctp.form.util.Enums.FieldAccessType;
import com.seeyon.ctp.form.util.Enums.FormResourcePropertyTypeEnum;
import com.seeyon.ctp.form.util.Enums.FormType;
import com.seeyon.ctp.form.util.infopath.ElementUtil;
import com.seeyon.ctp.form.util.infopath.InfoPathObject;
import com.seeyon.ctp.form.util.infopath.InfoPath_xsl;
import com.seeyon.ctp.organization.OrgConstants;
import com.seeyon.ctp.organization.bo.V3xOrgAccount;
import com.seeyon.ctp.organization.bo.V3xOrgMember;
import com.seeyon.ctp.organization.manager.OrgManager;
import com.seeyon.ctp.privilege.bo.PrivMenuBO;
import com.seeyon.ctp.privilege.bo.PrivTreeNodeBO;
import com.seeyon.ctp.privilege.dao.PrivilegeCacheImpl;
import com.seeyon.ctp.privilege.enums.MenuTypeEnums;
import com.seeyon.ctp.privilege.manager.PrivilegeMenuManager;
import com.seeyon.ctp.privilege.po.PrivRoleMenu;
import com.seeyon.ctp.util.DateUtil;
import com.seeyon.ctp.util.FlipInfo;
import com.seeyon.ctp.util.JDBCAgent;
import com.seeyon.ctp.util.Strings;
import com.seeyon.ctp.util.UUIDLong;
import com.seeyon.ctp.util.json.JSONUtil;
import com.seeyon.v3x.edoc.dao.EdocFormElementDao;
import com.seeyon.v3x.edoc.dao.UpgradeDao;
import com.seeyon.v3x.edoc.domain.EdocElementFlowPermAcl;
import com.seeyon.v3x.edoc.domain.EdocForm;
import com.seeyon.v3x.edoc.domain.EdocFormElement;
import com.seeyon.v3x.edoc.domain.EdocFormExtendInfo;
import com.seeyon.v3x.edoc.domain.EdocFormFlowPermBound;
import com.seeyon.v3x.edoc.domain.EdocRegister;
import com.seeyon.v3x.edoc.domain.EdocSummary;
import com.seeyon.v3x.edoc.webmodel.FormBoundPerm;
import com.seeyon.v3x.edoc.webmodel.FormOpinionConfig;
import com.seeyon.v3x.exchange.domain.EdocRecieveRecord;
import com.seeyon.v3x.exchange.domain.EdocSendRecord;

import www.seeyon.com.biz.enums.BizOperationEnum;
import www.seeyon.com.utils.FileUtil;

public class EdocUpgradeManagerImpl implements EdocUpgradeManager{
	
	private  UpgradeDao edocUpgradeDao;
	private  OrgManager orgManager;
	private  V3XFileDAO v3xFileDAO;
	private  FileManager fileManager;
	private  FormApi4Cap3 formApi4Cap3;
	private  GovdocFormExtendManager govdocFormExtendManager;
	private  GovdocFormOpinionSortManager govdocFormOpinionSortManager;
	private  EdocFormElementDao edocFormElementDao;
	private  EdocFormManager edocFormManager;
	private  EdocElementFlowPermAclManager edocElementFlowPermAclManager;

	private  PrivilegeMenuManager  privilegeMenuManager;
	private  AffairManager affairManager;
	private  PrivilegeCacheImpl privilegeCache;
	private ConfigManager configManager;
	private PartitionManager   partitionManager;
	
	public void setPartitionManager(PartitionManager partitionManager) {
		this.partitionManager = partitionManager;
	}
	public PrivilegeCacheImpl getPrivilegeCache() {
		return privilegeCache;
	}
	public void setPrivilegeCache(PrivilegeCacheImpl privilegeCache) {
		this.privilegeCache = privilegeCache;
	}

	private static Log LOG = CtpLogFactory.getLog(EdocUpgradeManagerImpl.class);
	public boolean NO_NEEDTO_CHECKED = false;//是否不需要检查升级
	
	//所有的发文种类升级，老数据的id，对应新数据的对象。
	Map<Long, CtpTemplateCategory> categories = new HashMap<Long, CtpTemplateCategory>();
	//保存老的文单，对应的新的文单。 键值分别是老文单id和新文单id
	private final Map<Long, Long> formIds = new HashMap<Long, Long>();
	//保存所有的单位
	List<V3xOrgAccount> units = new ArrayList<V3xOrgAccount>();	
	//保存预置表单数据
	private List<Long> presetForms= new ArrayList<Long>();
	//保存新增的公文元素
	List<Long> edocElementIds = new ArrayList<Long>();
	//新增的角色的id
	List<Long> saveRoleIds = new ArrayList<Long>();
		
	@Override
	public synchronized String upgrade() throws Exception {
		if ((Boolean) SysFlag.sys_isGovVer.getFlag()) {//G6升级
			return upgradeG6();
		} else {//A8/A6升级
			return upgradeA8();
		}
	}
	
	private String upgradeA8() throws Exception {
		SimpleDateFormat f = new SimpleDateFormat("yyyy-MM-dd hh:mm:ss");
		LOG.info("升级时间显示1"+f.format(new Date()));
		//这里的注释必须去掉，因为整个程序只能执行一次 
		//jdqxUpgrade(); 这个的升级迁移到升级程序里面去了
		try {
			units = orgManager.getAllAccounts();
			LOG.info("升级时间显示2"+f.format(new Date()));
			//公文元素添加
			insterEdocElements();
			LOG.info("升级时间显示3"+f.format(new Date()));
			//表单升级
			createFormByEdocForm();
			LOG.info("升级时间显示4"+f.format(new Date()));
			//预置表单数据
			presetgovdocForms();
			LOG.info("升级时间显示5"+f.format(new Date()));
			//模板升级
			LOG.info("模板升级开始...");
			edocUpgradeDao.templateUpgrade(formIds,categories);
			LOG.info("升级时间显示6"+f.format(new Date()));
			LOG.info("模板升级结束...");
			//兼容老公文数据  数据拷贝
			try{
				LOG.info("公文数据升级开始...copyRegisterEdocSummaryAndAffair");
				copyRegisterEdocSummaryAndAffair(); 
				LOG.info("公文数据升级开始...copyExchangeSendToEdocSummary");
				copyExchangeSendToEdocSummary();
				LOG.info("公文数据升级开始...copyExchangeRecToEdocSummary");
				copyExchangeRecToEdocSummary();
				
				LOG.info("公文数据升级开始...公文交换数据升级");
				updateExchangeData();
				LOG.info("公文数据升级开始...公文运行中流程数据升级开始");
				updateProcessRunningData();
				LOG.info("升级时间显示7"+f.format(new Date()));
				LOG.info("公文数据升级开始...公文运行中流程数据升级结束");
			}catch (Exception e) {
				LOG.error("复制交换和登记表数据出错：========="+e.getMessage(),e);
			}
			
			//升级form表单  receive_unit  sql
			try{
				LOG.info("升级表单 receive_unit 字段数据开始...");
				executeFormmainUpLevel();
				LOG.info("升级时间显示11"+f.format(new Date()));
				LOG.info("升级表单 receive_unit 字段数据结束...");
			}catch (Exception e) {
				LOG.error("升级表单 receive_unit 字段数据出错：========="+e.getMessage(),e);
			}
			LOG.info("升级时间显示10"+f.format(new Date()));
		} catch (Exception e) {
			LOG.error("公文升级出现异常：========="+e.getMessage(),e);
			throw e;
		}finally{
			//保存新老数据关系
			saveDataRelation();
			LOG.info("升级时间显示9"+f.format(new Date()));
		}
		return "-1";
	}
	
	private String upgradeG6() throws Exception {
		SimpleDateFormat f = new SimpleDateFormat("yyyy-MM-dd hh:mm:ss");
		LOG.info("升级时间显示1"+f.format(new Date()));
		//这里的注释必须去掉，因为整个程序只能执行一次 
		//jdqxUpgrade(); 这个的升级迁移到升级程序里面去了
		try {
			units = orgManager.getAllAccounts();
			LOG.info("升级时间显示2"+f.format(new Date()));
			//公文元素添加
			insterEdocElements();
			LOG.info("升级时间显示3"+f.format(new Date()));
			//表单升级
			createFormByEdocForm();
			LOG.info("升级时间显示4"+f.format(new Date()));
			//预置表单数据
			presetgovdocForms();
			LOG.info("升级时间显示5"+f.format(new Date()));
			//模板升级
			LOG.info("模板升级开始...");
			edocUpgradeDao.templateUpgrade(formIds,categories);
			LOG.info("升级时间显示6"+f.format(new Date()));
			LOG.info("模板升级结束...");
			//兼容老公文数据  数据拷贝
			try{
				LOG.info("公文数据升级开始...copyRegisterEdocSummaryAndAffair");
				copyRegisterEdocSummaryAndAffair(); 
				LOG.info("公文数据升级开始...copyExchangeSendToEdocSummary");
				copyExchangeSendToEdocSummary();
				LOG.info("公文数据升级开始...copyExchangeRecToEdocSummary");
				copyExchangeRecToEdocSummary();
				
				LOG.info("公文数据升级开始...公文交换数据升级");
				updateExchangeData();
				LOG.info("公文数据升级开始...公文运行中流程数据升级开始");
				updateProcessRunningData();
				LOG.info("升级时间显示7"+f.format(new Date()));
				LOG.info("公文数据升级开始...公文运行中流程数据升级结束");
			}catch (Exception e) {
				LOG.error("复制交换和登记表数据出错：========="+e.getMessage(),e);
			}
			
			//升级form表单  receive_unit  sql
			try{
				LOG.info("升级表单 receive_unit 字段数据开始...");
				executeFormmainUpLevel();
				LOG.info("升级时间显示11"+f.format(new Date()));
				LOG.info("升级表单 receive_unit 字段数据结束...");
			}catch (Exception e) {
				LOG.error("升级表单 receive_unit 字段数据出错：========="+e.getMessage(),e);
			}
			LOG.info("升级时间显示10"+f.format(new Date()));
		} catch (Exception e) {
			LOG.error("公文升级出现异常：========="+e.getMessage(),e);
			throw e;
		}finally{
			//保存新老数据关系
			saveDataRelation();
			LOG.info("升级时间显示9"+f.format(new Date()));
		}
		return "-1";
	}
	
	private void updateExchangeData() {

		String getdbType = getdbType();
		LOG.info("getdbType="+getdbType);
		if("MySQL".equalsIgnoreCase(getdbType)){
			List<String>  batchedSql = new ArrayList<String>();
			//-- 公文交换列表兼容老公文已分送数据
			String sql1 ="UPDATE EDOC_SUMMARY S, EDOC_EXCHANGE_SEND EX, CTP_AFFAIR A SET S.EXCHANGE_SEND_AFFAIRID = A.ID"+
			" WHERE S.ID = EX.ID AND EX.STATUS = 1 AND A.SUB_OBJECT_ID = EX.ID AND A.OBJECT_ID = EX.EDOC_ID"+
			" AND A.APP = 4 AND A.SUB_APP = 22 AND A.STATE = 4 AND S.EDOC_TYPE = 99 AND EX.SEND_USER_ID = A.MEMBER_ID ";
			batchedSql.add(sql1);
			//-- 老公文交换已发送、已回退数据同步到待办状态
	        String sql2 ="UPDATE CTP_AFFAIR A INNER JOIN EDOC_EXCHANGE_SEND EX ON A.SUB_OBJECT_ID = EX.ID AND EX.STATUS IN (1,2) SET A.STATE = 4"+
			" WHERE A.APP = 4 AND A.SUB_APP = 22 AND A.IS_DELETE = 0 AND A.STATE = 3";
	        batchedSql.add(sql2);
			//-- 老公文交换已签收、已登记、已回退数据同步到待办状态
			String sql3 ="UPDATE CTP_AFFAIR A INNER JOIN EDOC_EXCHANGE_RECIEVE EX ON A.SUB_OBJECT_ID = EX.ID AND EX.STATUS IN (1,2,3) SET A.STATE = 4"+
			" WHERE A.APP = 4 AND A.SUB_APP = 23 AND A.IS_DELETE = 0 AND A.STATE = 3";
			batchedSql.add(sql3);
			//-- 老公文交换已登记数据同步到待办状态
			String sql4 ="UPDATE CTP_AFFAIR A INNER JOIN EDOC_EXCHANGE_RECIEVE EX ON A.SUB_OBJECT_ID = EX.ID AND EX.STATUS = 2 SET A.STATE = 4"+
			" WHERE A.APP = 4 AND A.SUB_APP = 24 AND A.IS_DELETE = 0 AND A.STATE = 3";
			batchedSql.add(sql4);
			//-- 老公文交换已登记数据同步到待办状态
			String sql5 = "UPDATE CTP_AFFAIR A INNER JOIN EDOC_REGISTER ER ON A.OBJECT_ID = ER.ID AND ER.STATE = 2 SET A.STATE = 4"+
			" WHERE A.APP = 4 AND A.SUB_APP = 24 AND A.IS_DELETE = 0 AND A.STATE = 3";
			batchedSql.add(sql5);
			//-- 老公文交换已分发数据同步到待办状态
			String sql6 = "UPDATE CTP_AFFAIR A INNER JOIN EDOC_REGISTER ER ON A.OBJECT_ID = ER.ID AND ER.DISTRIBUTE_STATE = 2 SET A.STATE = 4"+
			" WHERE A.APP = 4 AND A.SUB_APP = 34 AND A.IS_DELETE = 0 AND A.STATE = 3";
			batchedSql.add(sql6);
			
			//-- 转发文数据升级
			String sql7 = "INSERT INTO GOVDOC_EXCHANGE_MAIN ( ID, SUBJECT, CREATE_TIME, SUMMARY_ID, EXCHANGE_TYPE, REFERENCE_ID)"+
			" SELECT ID, SUBJECT, TURN_DATE, SUMMARY_ID, 3, REFERENCE FROM GOVDOC_RELATION WHERE GOVDOC_TYPE = 1";
			batchedSql.add(sql7);
			//-- 升级EDOC_SUMMARY字段EXCHANGE_TYPE
			String sql8 = "UPDATE EDOC_SUMMARY A, GOVDOC_EXCHANGE_MAIN B,GOVDOC_EXCHANGE_DETAIL C SET A.EXCHANGE_TYPE = B.EXCHANGE_TYPE"+
			" WHERE C.MAIN_ID = B.ID AND C.SUMMARY_ID = A.ID";
			batchedSql.add(sql8);
			//-- 设置交换类型默认值
			String sql9 = "UPDATE EDOC_SUMMARY SET EXCHANGE_TYPE = 0 WHERE EXCHANGE_TYPE IS NULL";
			batchedSql.add(sql9);
			//-- 老公文回退流转状态维护
			String sql10 = "UPDATE EDOC_SUMMARY A, GOVDOC_EXCHANGE_DETAIL_LOG B SET A.TRANSFER_STATUS = 8 WHERE A.ID = B.BACK_SUMMARY_ID AND B.STATUS = 10";
			batchedSql.add(sql10);
			//-- 修改退文时间 
			String sql11 = "UPDATE EDOC_SUMMARY E, GOVDOC_EXCHANGE_DETAIL_LOG L SET E.COMPLETE_TIME = L.CREATE_TIME"+ 
			" WHERE L.BACK_SUMMARY_ID = E.ID AND E.TRANSFER_STATUS = 8 AND E.COMPLETE_TIME IS NULL";
			batchedSql.add(sql11);
			
			//-- 已签收的数据修改删除状态
			String sql12 = "update ctp_affair affair,edoc_exchange_recieve recieve set affair.is_delete = 0"+
			" where affair.SUB_OBJECT_ID = recieve.id and affair.state = 4 and recieve.status in (1,2,10)";
			batchedSql.add(sql12);
			//-- 已发送的数据修改删除状态
			String sql13 = "update ctp_affair affair,edoc_exchange_send send set affair.is_delete = 0"+
			" where affair.SUB_OBJECT_ID = send.id and affair.state = 4 and send.status = 1";
			batchedSql.add(sql13);
			
			JDBCAgent jdbc = new JDBCAgent();
			try {
				jdbc.executeBatch(batchedSql);
			} catch (BusinessException e) {
				LOG.error("升级公文交换数据报错1");
			} catch (SQLException e) {
				LOG.error("升级公文交换数据报错2");
			}
			
		}else if("Oracle".equalsIgnoreCase(getdbType)){
			List<String>  batchedSql = new ArrayList<String>();
			//-- 公文交换列表兼容老公文已分送数据
			String sql1 = "UPDATE EDOC_SUMMARY S SET S.EXCHANGE_SEND_AFFAIRID = ( SELECT A.ID FROM CTP_AFFAIR A, EDOC_EXCHANGE_SEND EX WHERE S.ID = EX.ID AND EX.STATUS = 1"+
			" AND A.SUB_OBJECT_ID = EX.ID AND A.OBJECT_ID = EX.EDOC_ID AND A.APP = 4 AND A.SUB_APP = 22 AND A.STATE = 4 AND EX.SEND_USER_ID = A.MEMBER_ID )"+
			" WHERE S.EDOC_TYPE = 99 AND S.EXCHANGE_SEND_AFFAIRID IS NULL AND EXISTS ( SELECT A.ID FROM CTP_AFFAIR A, EDOC_EXCHANGE_SEND EX WHERE S.ID = EX.ID AND EX.STATUS = 1"+
			" AND A.SUB_OBJECT_ID = EX.ID AND A.OBJECT_ID = EX.EDOC_ID AND A.APP = 4 AND A.SUB_APP = 22 AND A.STATE = 4 AND EX.SEND_USER_ID = A.MEMBER_ID)";
			batchedSql.add(sql1);
			//-- 老公文交换已发送、已回退数据同步到待办状态
			String sql2 = "UPDATE CTP_AFFAIR A SET A.STATE = 4 WHERE A.APP = 4 AND A.SUB_APP = 22 AND A.IS_DELETE = 0 AND A.STATE = 3"+
			" AND A.SUB_OBJECT_ID = (SELECT EX.ID FROM EDOC_EXCHANGE_SEND EX WHERE A.SUB_OBJECT_ID = EX.ID AND EX.STATUS IN (1,2))"+
			" AND EXISTS (SELECT EX.ID FROM EDOC_EXCHANGE_SEND EX WHERE A.SUB_OBJECT_ID = EX.ID AND EX.STATUS IN (1,2))";
			batchedSql.add(sql2);
			//-- 老公文交换已签收、已登记、已回退数据同步到待办状态
			String sql3 = "UPDATE CTP_AFFAIR A SET A.STATE = 4 WHERE A.APP = 4 AND A.SUB_APP = 23 AND A.IS_DELETE = 0 AND A.STATE = 3"+
			" AND A.SUB_OBJECT_ID = (SELECT EX.ID FROM EDOC_EXCHANGE_RECIEVE EX WHERE A.SUB_OBJECT_ID = EX.ID AND EX.STATUS IN (1,2,3))"+
			" AND EXISTS (SELECT EX.ID FROM EDOC_EXCHANGE_RECIEVE EX WHERE A.SUB_OBJECT_ID = EX.ID AND EX.STATUS IN (1,2,3))";
			batchedSql.add(sql3);
			//-- 老公文交换已登记数据同步到待办状态
			String sql4 ="UPDATE CTP_AFFAIR A SET A.STATE = 4 WHERE A.APP = 4 AND A.SUB_APP = 24 AND A.IS_DELETE = 0 AND A.STATE = 3"+
			" AND A.SUB_OBJECT_ID = (SELECT EX.ID FROM EDOC_EXCHANGE_RECIEVE EX WHERE A.SUB_OBJECT_ID = EX.ID AND EX.STATUS = 2)"+
			" AND EXISTS (SELECT EX.ID FROM EDOC_EXCHANGE_RECIEVE EX WHERE A.SUB_OBJECT_ID = EX.ID AND EX.STATUS = 2)";
			batchedSql.add(sql4);
			//-- 老公文交换已登记数据同步到待办状态
			String sql5 = "UPDATE CTP_AFFAIR A SET A.STATE = 4 WHERE A.APP = 4 AND A.SUB_APP = 24 AND A.IS_DELETE = 0 AND A.STATE = 3"+
			" AND A.OBJECT_ID = (SELECT ER.ID FROM EDOC_REGISTER ER WHERE A.OBJECT_ID = ER.ID AND ER.STATE = 2)"+
			" AND EXISTS (SELECT ER.ID FROM EDOC_REGISTER ER WHERE A.OBJECT_ID = ER.ID AND ER.STATE = 2)";
			batchedSql.add(sql5);
			//-- 老公文交换已分发数据同步到待办状态
			String sql6 = "UPDATE CTP_AFFAIR A SET A.STATE = 4 WHERE A.APP = 4 AND A.SUB_APP = 34 AND A.IS_DELETE = 0 AND A.STATE = 3"+
			" AND A.OBJECT_ID = (SELECT ER.ID FROM EDOC_REGISTER ER WHERE A.OBJECT_ID = ER.ID AND ER.DISTRIBUTE_STATE = 2)"+
			" AND EXISTS (SELECT ER.ID FROM EDOC_REGISTER ER WHERE A.OBJECT_ID = ER.ID AND ER.DISTRIBUTE_STATE = 2)";
			batchedSql.add(sql6);
			
			//-- 转发文数据升级
			String sql7 = "INSERT INTO GOVDOC_EXCHANGE_MAIN ( ID, SUBJECT, CREATE_TIME, SUMMARY_ID, EXCHANGE_TYPE, REFERENCE_ID)"+
			" SELECT ID, SUBJECT, TURN_DATE, SUMMARY_ID, 3, REFERENCE FROM GOVDOC_RELATION WHERE GOVDOC_TYPE = 1";
			batchedSql.add(sql7);
			//-- 升级EDOC_SUMMARY字段EXCHANGE_TYPE
			String sql8 = "UPDATE EDOC_SUMMARY A SET A.EXCHANGE_TYPE = (SELECT EXCHANGE_TYPE FROM GOVDOC_EXCHANGE_MAIN B,GOVDOC_EXCHANGE_DETAIL C WHERE C.MAIN_ID = B.ID AND C.SUMMARY_ID = A.ID)";
			batchedSql.add(sql8);
			//-- 设置交换类型默认值
			String sql9 = "UPDATE EDOC_SUMMARY SET EXCHANGE_TYPE = 0 WHERE EXCHANGE_TYPE IS NULL";
			batchedSql.add(sql9);
			//-- 老公文回退流转状态维护
			String sql10 = "UPDATE EDOC_SUMMARY A SET A.TRANSFER_STATUS = 8 WHERE A.ID IN (SELECT BACK_SUMMARY_ID FROM GOVDOC_EXCHANGE_DETAIL_LOG WHERE STATUS = 10)";
			batchedSql.add(sql10);
			//-- 修改退文时间
			String sql11 = "UPDATE EDOC_SUMMARY E SET COMPLETE_TIME = (SELECT MAX(CREATE_TIME) FROM GOVDOC_EXCHANGE_DETAIL_LOG L WHERE L.BACK_SUMMARY_ID = E.ID)"+
			" WHERE E.TRANSFER_STATUS = 8 AND E.COMPLETE_TIME IS NULL";
			batchedSql.add(sql11);
			
			//-- 已签收的数据修改删除状态
			String sql12 = "update ctp_affair affair set affair.is_delete = 0 where exists (select recieve.id from edoc_exchange_recieve recieve"+
			" where affair.SUB_OBJECT_ID = recieve.id and affair.state = 4 and recieve.status in (1,2,10))";
			batchedSql.add(sql12);
			//-- 已发送的数据修改删除状态
			String sql13 ="update ctp_affair affair set affair.is_delete = 0 where exists (select send.id from edoc_exchange_send send"+
			" where affair.SUB_OBJECT_ID = send.id and affair.state = 4 and send.status = 1)";
			batchedSql.add(sql13);
			
			JDBCAgent jdbc = new JDBCAgent();
			try {
				jdbc.executeBatch(batchedSql);
			} catch (BusinessException e) {
				LOG.error("升级公文交换数据报错1");
			} catch (SQLException e) {
				LOG.error("升级公文交换数据报错2");
			}
			
		}else if("postgresql".equalsIgnoreCase(getdbType)){
			List<String>  batchedSql = new ArrayList<String>();
			//-- 公文交换列表兼容老公文已分送数据
			String sql1 = "UPDATE EDOC_SUMMARY S SET EXCHANGE_SEND_AFFAIRID = ( SELECT A.ID FROM CTP_AFFAIR A, EDOC_EXCHANGE_SEND EX WHERE S.ID = EX.ID AND EX.STATUS = 1"+
			" AND A.SUB_OBJECT_ID = EX.ID AND A.OBJECT_ID = EX.EDOC_ID AND A.APP = 4 AND A.SUB_APP = 22 AND A.STATE = 4 AND EX.SEND_USER_ID = A.MEMBER_ID )"+
			" WHERE S.EDOC_TYPE = 99 AND S.EXCHANGE_SEND_AFFAIRID IS NULL AND EXISTS ( SELECT A.ID FROM CTP_AFFAIR A, EDOC_EXCHANGE_SEND EX WHERE S.ID = EX.ID AND EX.STATUS = 1"+
			" AND A.SUB_OBJECT_ID = EX.ID AND A.OBJECT_ID = EX.EDOC_ID AND A.APP = 4 AND A.SUB_APP = 22 AND A.STATE = 4 AND EX.SEND_USER_ID = A.MEMBER_ID)";
			batchedSql.add(sql1);
			//-- 老公文交换已发送、已回退数据同步到待办状态
			String sql2 = "UPDATE CTP_AFFAIR A SET STATE = 4 WHERE A.APP = 4 AND A.SUB_APP = 22 AND A.IS_DELETE = 0 AND A.STATE = 3"+
			" AND A.SUB_OBJECT_ID = (SELECT EX.ID FROM EDOC_EXCHANGE_SEND EX WHERE A.SUB_OBJECT_ID = EX.ID AND EX.STATUS IN (1,2))"+
			" AND EXISTS (SELECT EX.ID FROM EDOC_EXCHANGE_SEND EX WHERE A.SUB_OBJECT_ID = EX.ID AND EX.STATUS IN (1,2))";
			batchedSql.add(sql2);
			//-- 老公文交换已签收、已登记、已回退数据同步到待办状态
			String sql3 = "UPDATE CTP_AFFAIR A SET STATE = 4 WHERE A.APP = 4 AND A.SUB_APP = 23 AND A.IS_DELETE = 0 AND A.STATE = 3"+
			" AND A.SUB_OBJECT_ID = (SELECT EX.ID FROM EDOC_EXCHANGE_RECIEVE EX WHERE A.SUB_OBJECT_ID = EX.ID AND EX.STATUS IN (1,2,3))"+
			" AND EXISTS (SELECT EX.ID FROM EDOC_EXCHANGE_RECIEVE EX WHERE A.SUB_OBJECT_ID = EX.ID AND EX.STATUS IN (1,2,3))";
			batchedSql.add(sql3);
			//-- 老公文交换已登记数据同步到待办状态
			String sql4 ="UPDATE CTP_AFFAIR A SET STATE = 4 WHERE A.APP = 4 AND A.SUB_APP = 24 AND A.IS_DELETE = 0 AND A.STATE = 3"+
			" AND A.SUB_OBJECT_ID = (SELECT EX.ID FROM EDOC_EXCHANGE_RECIEVE EX WHERE A.SUB_OBJECT_ID = EX.ID AND EX.STATUS = 2)"+
			" AND EXISTS (SELECT EX.ID FROM EDOC_EXCHANGE_RECIEVE EX WHERE A.SUB_OBJECT_ID = EX.ID AND EX.STATUS = 2)";
			batchedSql.add(sql4);
			//-- 老公文交换已登记数据同步到待办状态
			String sql5 = "UPDATE CTP_AFFAIR A SET STATE = 4 WHERE A.APP = 4 AND A.SUB_APP = 24 AND A.IS_DELETE = 0 AND A.STATE = 3"+
			" AND A.OBJECT_ID = (SELECT ER.ID FROM EDOC_REGISTER ER WHERE A.OBJECT_ID = ER.ID AND ER.STATE = 2)"+
			" AND EXISTS (SELECT ER.ID FROM EDOC_REGISTER ER WHERE A.OBJECT_ID = ER.ID AND ER.STATE = 2)";
			batchedSql.add(sql5);
			//-- 老公文交换已分发数据同步到待办状态
			String sql6 = "UPDATE CTP_AFFAIR A SET STATE = 4 WHERE A.APP = 4 AND A.SUB_APP = 34 AND A.IS_DELETE = 0 AND A.STATE = 3"+
			" AND A.OBJECT_ID = (SELECT ER.ID FROM EDOC_REGISTER ER WHERE A.OBJECT_ID = ER.ID AND ER.DISTRIBUTE_STATE = 2)"+
			" AND EXISTS (SELECT ER.ID FROM EDOC_REGISTER ER WHERE A.OBJECT_ID = ER.ID AND ER.DISTRIBUTE_STATE = 2)";
			batchedSql.add(sql6);
			
			//-- 转发文数据升级
			String sql7 = "INSERT INTO GOVDOC_EXCHANGE_MAIN ( ID, SUBJECT, CREATE_TIME, SUMMARY_ID, EXCHANGE_TYPE, REFERENCE_ID)"+
			" SELECT ID, SUBJECT, TURN_DATE, SUMMARY_ID, 3, REFERENCE FROM GOVDOC_RELATION WHERE GOVDOC_TYPE = 1";
			batchedSql.add(sql7);
			//-- 升级EDOC_SUMMARY字段EXCHANGE_TYPE
			String sql8 = "UPDATE EDOC_SUMMARY A SET EXCHANGE_TYPE = (SELECT EXCHANGE_TYPE FROM GOVDOC_EXCHANGE_MAIN B,GOVDOC_EXCHANGE_DETAIL C WHERE C.MAIN_ID = B.ID AND C.SUMMARY_ID = A.ID)";
			batchedSql.add(sql8);
			//-- 设置交换类型默认值
			String sql9 = "UPDATE EDOC_SUMMARY SET EXCHANGE_TYPE = 0 WHERE EXCHANGE_TYPE IS NULL";
			batchedSql.add(sql9);
			//-- 老公文回退流转状态维护
			String sql10 = "UPDATE EDOC_SUMMARY A SET TRANSFER_STATUS = 8 WHERE A.ID IN (SELECT BACK_SUMMARY_ID FROM GOVDOC_EXCHANGE_DETAIL_LOG WHERE STATUS = 10)";
			batchedSql.add(sql10);
			//-- 修改退文时间
			String sql11 = "UPDATE EDOC_SUMMARY E SET COMPLETE_TIME = (SELECT MAX(CREATE_TIME) FROM GOVDOC_EXCHANGE_DETAIL_LOG L WHERE L.BACK_SUMMARY_ID = E.ID)"+
			" WHERE E.TRANSFER_STATUS = 8 AND E.COMPLETE_TIME IS NULL";
			batchedSql.add(sql11);
			
			//-- 已签收的数据修改删除状态
			String sql12 = "update ctp_affair affair set is_delete = 0 where exists (select recieve.id from edoc_exchange_recieve recieve"+
			" where affair.SUB_OBJECT_ID = recieve.id and affair.state = 4 and recieve.status in (1,2,10))";
			batchedSql.add(sql12);
			//-- 已发送的数据修改删除状态
			String sql13 ="update ctp_affair affair set is_delete = 0 where exists (select send.id from edoc_exchange_send send"+
			" where affair.SUB_OBJECT_ID = send.id and affair.state = 4 and send.status = 1)";
			batchedSql.add(sql13);
			
			JDBCAgent jdbc = new JDBCAgent();
			try {
				jdbc.executeBatch(batchedSql);
			} catch (BusinessException e) {
				LOG.error("升级公文交换数据报错1");
			} catch (SQLException e) {
				LOG.error("升级公文交换数据报错2");
			}
			
		}
		else if("Microsoft SQL Server".equalsIgnoreCase(getdbType)){
			List<String>  batchedSql = new ArrayList<String>();
			//-- 公文交换列表兼容老公文已分送数据
			String sql1 = "UPDATE S SET S.EXCHANGE_SEND_AFFAIRID = A.ID FROM EDOC_SUMMARY S INNER JOIN EDOC_EXCHANGE_SEND EX ON S.ID = EX.ID AND EX.STATUS = 1"+
			" INNER JOIN CTP_AFFAIR A ON A.SUB_OBJECT_ID = EX.ID AND A.OBJECT_ID = EX.EDOC_ID WHERE A.APP = 4 AND A.SUB_APP = 22 AND A.STATE = 4"+
			" AND S.EDOC_TYPE = 99 AND EX.SEND_USER_ID = A.MEMBER_ID";
			batchedSql.add(sql1);
			//-- 老公文交换已发送、已回退数据同步到待办状态
			String sql2= "UPDATE A SET A.STATE = 4 FROM CTP_AFFAIR A INNER JOIN EDOC_EXCHANGE_SEND EX ON A.SUB_OBJECT_ID = EX.ID AND EX.STATUS IN (1,2)"+
			" WHERE A.APP = 4 AND A.SUB_APP = 22 AND A.IS_DELETE = 0 AND A.STATE = 3";
			batchedSql.add(sql2);
			//-- 老公文交换已签收、已登记、已回退数据同步到待办状态
			String sql3 = "UPDATE A SET A.STATE = 4 FROM CTP_AFFAIR A INNER JOIN EDOC_EXCHANGE_RECIEVE EX ON A.SUB_OBJECT_ID = EX.ID AND EX.STATUS IN (1,2,3)"+
			" WHERE A.APP = 4 AND A.SUB_APP = 23 AND A.IS_DELETE = 0 AND A.STATE = 3";
			batchedSql.add(sql3);
			//-- 老公文交换已登记数据同步到待办状态
			String sql4 = "UPDATE A SET A.STATE = 4 FROM CTP_AFFAIR A INNER JOIN EDOC_EXCHANGE_RECIEVE EX ON A.SUB_OBJECT_ID = EX.ID AND EX.STATUS = 2"+
			" WHERE A.APP = 4 AND A.SUB_APP = 24 AND A.IS_DELETE = 0 AND A.STATE = 3";
			batchedSql.add(sql4);
			//-- 老公文交换已登记数据同步到待办状态
			String sql5 = "UPDATE A SET A.STATE = 4 FROM CTP_AFFAIR A INNER JOIN EDOC_REGISTER ER ON A.OBJECT_ID = ER.ID AND ER.STATE = 2"+
			" WHERE A.APP = 4 AND A.SUB_APP = 24 AND A.IS_DELETE = 0 AND A.STATE = 3";
			batchedSql.add(sql5);
			//-- 老公文交换已分发数据同步到待办状态
			String sql6 = "UPDATE A SET A.STATE = 4 FROM CTP_AFFAIR A INNER JOIN EDOC_REGISTER ER ON A.OBJECT_ID = ER.ID AND ER.DISTRIBUTE_STATE = 2"+
			" WHERE A.APP = 4 AND A.SUB_APP = 34 AND A.IS_DELETE = 0 AND A.STATE = 3";
			batchedSql.add(sql6);
			
			//-- 转发文数据升级
			String sql7 = "INSERT INTO GOVDOC_EXCHANGE_MAIN ( ID, SUBJECT, CREATE_TIME, SUMMARY_ID, EXCHANGE_TYPE, REFERENCE_ID)"+ 
			" SELECT ID, SUBJECT, TURN_DATE, SUMMARY_ID, 3, REFERENCE FROM GOVDOC_RELATION WHERE GOVDOC_TYPE = 1";
			batchedSql.add(sql7);
			//-- 升级EDOC_SUMMARY字段EXCHANGE_TYPE
			String sql8 = "UPDATE A SET A.EXCHANGE_TYPE = B.EXCHANGE_TYPE FROM EDOC_SUMMARY A, GOVDOC_EXCHANGE_MAIN B,GOVDOC_EXCHANGE_DETAIL C"+
			" WHERE C.MAIN_ID = B.ID AND C.SUMMARY_ID = A.ID";
			batchedSql.add(sql8);
			//-- 设置交换类型默认值
			String sql9 = "UPDATE EDOC_SUMMARY SET EXCHANGE_TYPE = 0 WHERE EXCHANGE_TYPE IS NULL";
			batchedSql.add(sql9);
			//-- 老公文回退流转状态维护
			String sql10 = "UPDATE A SET A.TRANSFER_STATUS=8 FROM EDOC_SUMMARY A, GOVDOC_EXCHANGE_DETAIL_LOG B WHERE A.ID = B.BACK_SUMMARY_ID AND B.STATUS = 10";
			batchedSql.add(sql10);
			//-- 修改退文时间
			String sql11 = "UPDATE E SET E.COMPLETE_TIME = L.CREATE_TIME FROM EDOC_SUMMARY E, GOVDOC_EXCHANGE_DETAIL_LOG L"+ 
			" WHERE L.BACK_SUMMARY_ID = E.ID AND E.TRANSFER_STATUS = 8 AND E.COMPLETE_TIME IS NULL";
			batchedSql.add(sql11);
			
			//-- 已签收的数据修改删除状态
			String sql12="update affair set affair.is_delete = 0 from ctp_affair affair inner join edoc_exchange_recieve recieve on affair.sub_object_id=recieve.id"+
			" where affair.state = 4 and recieve.status in (1,2,10)";
			batchedSql.add(sql12);
			//-- 已发送的数据修改删除状态
			String sql13= "update affair set affair.is_delete = 0 from ctp_affair affair"+
			" inner join edoc_exchange_send send on affair.sub_object_id=send.id"+
			" where affair.state = 4 and send.status = 1";
			batchedSql.add(sql13);
			
			JDBCAgent jdbc = new JDBCAgent();
			try {
				jdbc.executeBatch(batchedSql);
			} catch (BusinessException e) {
				LOG.error("升级公文交换数据报错1");
			} catch (SQLException e) {
				LOG.error("升级公文交换数据报错2");
			}
			
		}
	
	}

	private void updateProcessRunningData(){
		FlipInfo flipInfo = new FlipInfo(0, -1);
		flipInfo.setNeedTotal(false);
		String hql  = "select wf.id ,wf.processobject,s.edoc_type from edoc_summary s, wf_process_running   wf where s.process_id = wf.id and s.state  = :state";
		String updateSql = "update wf_process_running set processobject = ? where id = ?";
		Map<String,Object> param = new HashMap<String, Object>();
		param.put("state", EdocConstant.flowState.run.ordinal());
		JDBCAgent jdbc = new JDBCAgent();
		
		try {
			
			jdbc.findNameByPaging(hql, param, flipInfo);
			List<Map<String,Object>> result = flipInfo.getData();
			if (Strings.isNotEmpty(result)) {
				jdbc.batch1Prepare(updateSql);
				List<Object> updateDataList;
				
				for (Map<String,Object> map : result) {
					String processXml = parseBpmProcess(map.get("processobject"));
					Long processId = Long.valueOf(map.get("id").toString());
					Object edocTypeObj = map.get("edoc_type");
					
					if(null==edocTypeObj){
						continue;
					}
					Integer edocType = Integer.valueOf(edocTypeObj.toString());
					if (Strings.isNotBlank(processXml) && processXml.contains("Departmentexchange")) {

						if (edocType == 0 || edocType == 2) {
							processXml = processXml.replaceAll("Departmentexchange", "DepartmentGovdocSend")
									.replaceAll("公文收发员", "公文送文员");
						} else if (edocType == 1) {
							processXml = processXml.replaceAll("Departmentexchange", "DepartmentGovdocRec")
									.replaceAll("公文收发员", "公文收文员");
						}
						updateDataList = new ArrayList<Object>();
						updateDataList.add(transBpmObjectToByte(processXml));
						updateDataList.add(processId);
						jdbc.batch2Add(updateDataList);
					}
				}

				jdbc.batch3Execute();
			} 
		} catch (Exception e) {
			LOG.error("", e);
		}finally {
			jdbc.close();
		}
		 
		
	}
	
	private byte[] transBpmObjectToByte(String xml){
		ByteArrayOutputStream byt = null;
		ObjectOutputStream oos = null;
		try {
			byt = new ByteArrayOutputStream();
			oos = new ObjectOutputStream(byt);
			oos.writeObject(xml);					
		} catch(IOException e) {
			LOG.error("", e);
		} finally {
			if(oos != null){
                try {
                	oos.close();
                } catch (IOException e) {
                	LOG.error("", e);
                }
            }
			if(byt != null){
                try {
                	byt.close();
                } catch (IOException e) {
                	LOG.error("", e);
                }
            }
		}
		byte[] bytes = byt.toByteArray();
		return bytes;
	}
	
	private String parseBpmProcess(Object caseobject) {
		byte[] byteArray = null;
		String dbms = JDBCAgent.getDBType();
		Object obj = null;
		InputStream is = null;
        ObjectInputStream ois = null;
        try {
			if ("MySQL".equalsIgnoreCase(dbms)) {
				byteArray = (byte[])caseobject;
				if (null != byteArray) {
					is = new ByteArrayInputStream(byteArray);
					ois = new ObjectInputStream(is);
	                obj = ois.readObject();
				}
			} else if ("Microsoft SQL Server".equalsIgnoreCase(dbms) || "sqlserver".equalsIgnoreCase(dbms)) {
				
				try{
					byteArray = (byte[])caseobject;
					if (null != byteArray) {
						is = new ByteArrayInputStream(byteArray);
						ois = new ObjectInputStream(is);
		                obj = ois.readObject();
					}
                }catch(Throwable e){
                    LOG.warn("blob大字段解析失败，兼容方式解析", e);
                    java.sql.Blob blob = (java.sql.Blob)caseobject;
                    is = blob.getBinaryStream();
                    ois = new ObjectInputStream(is);
                    obj = ois.readObject();
                }
				
				
				
			}else if("Oracle".equalsIgnoreCase(dbms)){
				
				java.sql.Blob blob = (java.sql.Blob)caseobject;
				is = blob.getBinaryStream();
				ois = new ObjectInputStream(is);
				obj = ois.readObject();
			}
		} catch (Throwable e) {
        	LOG.error(e.getMessage(), e);
        } finally {
            if(is != null){
                try {
                	is.close();
                } catch (IOException e) {
                	LOG.error("", e);
                }
            }
            if(ois != null){
                try {
                	ois.close();
                } catch (IOException e) {
                	LOG.error("", e);
                }
            }
        }
		return (String) obj;
	}
	
	
	
	private String getdbType(){
		String dbms = JDBCAgent.getDBType();
		return dbms;
	}
	
	public AffairManager getAffairManager() {
		return affairManager;
	}
	
	
	public void setAffairManager(AffairManager affairManager) {
		this.affairManager = affairManager;
	}
	
	private void copyRegisterEdocSummaryAndAffair(){
		String hql  = "from EdocRegister where state in (:state)";
		FlipInfo fileinfo = new FlipInfo();
		Map<String,Object> param = new HashMap<String, Object>();
		List<Integer> states = new ArrayList<Integer>();
		states.add(0);
		states.add(1);
		states.add(2);
		param.put("state", states);
		int page = 1;
		fileinfo.setSize(500);
		fileinfo.setPage(page);
		int rowsCount = 0;
		List<CtpAffair> affairs = new ArrayList<CtpAffair>();
		List<EdocSummary> summarys = new ArrayList<EdocSummary>();
		List<EdocRegister> edocSendRecords= (List<EdocRegister> )edocUpgradeDao.getDataByHql(hql,param ,fileinfo);
		rowsCount = fileinfo.getTotal();
		
		if(rowsCount == 0) {
			return;
		}
		
		
		do{
			LOG.info("正在操作<EdocRegister>第" + ((fileinfo.getPage() - 1) * fileinfo.getSize() + 1) + "条数据");
			for(EdocRegister record : edocSendRecords){
				//复制部分信息到summary中
				EdocSummary edocSummary = new EdocSummary();
				edocSummary.setId(record.getId());
				edocSummary.setSubject(record.getSubject());
				edocSummary.setEdocType(99);
				edocSummary.setDeadline(-1L);
				edocSummary.setHasArchive(false);
				edocSummary.setGovdocType(0);
				int state = record.getState();
				edocSummary.setState(7);//ColConstant.flowState.oldexchange.ordinal()
				edocSummary.setDocMark(record.getDocMark());
				edocSummary.setSecretLevel(record.getSecretLevel());
				edocSummary.setUrgentLevel(record.getUrgentLevel());
				edocSummary.setCreateTime(record.getCreateTime());
				edocSummary.setStartTime(record.getCreateTime());
				edocSummary.setCreatePerson(record.getRegisterUserName());
				edocSummary.setStartUserId(record.getRegisterUserId());
				if(record.getRegisterUserId() == null || "".equals(record.getRegisterUserId())){
					edocSummary.setCreatePerson(record.getDistributer());
					edocSummary.setStartUserId(record.getDistributerId());
				}
				edocSummary.setOrgAccountId(record.getSendUnitId());
				//将 edoc_id  写入 edoc_summary中的caseid  保存 没实际意义，只是为了前台 打开以前界面参数
				//量receive_id  写入 edoc_summary中的processid保存 没实际意义，只是为了前台 打开以前界面参数
				edocSummary.setCaseId(record.getEdocId());
				edocSummary.setProcessId(record.getRecieveId()+"");
				//复制部分信息到affair中   待登记的 不需要在affair中添加数据
				if(0 == state || 2 == state){
					CtpAffair affair = new CtpAffair();
					affair.setId(UUIDLong.longUUID());
					if(0 == state){
						affair.setState(3);
					}else if(2 == state){
						affair.setState(4);
						affair.setCompleteTime(record.getCreateTime());
						edocSummary.setCompleteTime(record.getCreateTime());
					}
					affair.setSubApp(24);
					affair.setSubject(record.getSubject());
					affair.setTrack(0);
					affair.setObjectId(record.getId());
					affair.setSubObjectId(record.getRecieveId());
					affair.setApp(4);//
					affair.setCreateDate(record.getCreateTime());
					affair.setMemberId(record.getDistributerId());
					affair.setSenderId(record.getDistributerId());
					if(StringUtils.isNotBlank(record.getUrgentLevel())){
						affair.setImportantLevel(Integer.valueOf(record.getUrgentLevel()));
					}
					affairs.add(affair);
				}
				summarys.add(edocSummary);
			}
			edocUpgradeDao.saveOrUpdateAll(summarys);
			edocUpgradeDao.saveAll(affairs);
			summarys.clear();
			affairs.clear();
			if(rowsCount <= page*500){
				break;
			}
			fileinfo.setPage(++page);
			fileinfo.setNeedTotal(false);
			edocSendRecords= (List<EdocRegister> )edocUpgradeDao.getDataByHql(hql,param ,fileinfo);
		}
		while(Strings.isNotEmpty(fileinfo.getData()) && fileinfo.getData().size() > 0);
	}
	private void copyExchangeRecToEdocSummary() throws BusinessException, SQLException{
		//该处只将待签收/已签收/已登记/登记待发的签收数据生成edoc_summary
		String hql  = "from EdocRecieveRecord where status in (0,1,2,10)";//待签收0/已签收1/已登记2/登记保存待发10/回退或取回3/被退回到退件箱4/已签收-被删除5/待签收-被删除6
		FlipInfo fileinfo = new FlipInfo();
		fileinfo.setSize(500);
		int page = 1;
		fileinfo.setPage(page);
		int rowsCount = 0;
		List<EdocSummary> summarys = new ArrayList<EdocSummary>();
		List<EdocRecieveRecord> edocSendRecords= (List<EdocRecieveRecord> )edocUpgradeDao.getDataByHql(hql,new HashMap<String, Object>() ,fileinfo);
		rowsCount = fileinfo.getTotal();
		
		if(rowsCount == 0) {
			return;
		}
		
		do{
			LOG.info("正在操作<EdocRecieveRecord>第" + ((fileinfo.getPage() - 1) * fileinfo.getSize() + 1) + "条数据");
			for(EdocRecieveRecord rec : edocSendRecords){
				EdocSummary edocSummary = new EdocSummary();
				edocSummary.setId(rec.getId());
				edocSummary.setSubject(rec.getSubject());
				edocSummary.setEdocType(99);
				edocSummary.setDeadline(-1L);
				edocSummary.setHasArchive(false);
				edocSummary.setGovdocType(0);
				edocSummary.setState(7);//ColConstant.flowState.oldexchange.ordinal()
				edocSummary.setDocMark(rec.getDocMark());
				edocSummary.setSecretLevel(rec.getSecretLevel());
				edocSummary.setUrgentLevel(rec.getUrgentLevel());
				edocSummary.setCreateTime(rec.getCreateTime());
				edocSummary.setCreatePerson(rec.getSender());
				edocSummary.setStartUserId(rec.getRecUserId());
				edocSummary.setOrgAccountId(rec.getExchangeOrgId());
				edocSummary.setStartTime(rec.getRecTime());
				summarys.add(edocSummary);
			}
			edocUpgradeDao.saveOrUpdateAll(summarys);
			summarys.clear();
			if(rowsCount <= page*500){
				break;
			}
			fileinfo.setPage(++page);
			fileinfo.setNeedTotal(false);
			edocSendRecords= (List<EdocRecieveRecord> )edocUpgradeDao.getDataByHql(hql,new HashMap<String, Object>() ,fileinfo);
		}while(Strings.isNotEmpty(fileinfo.getData()) && fileinfo.getData().size() > 0);
		
		//更新affair结束时间
		String updateSql = "update ctp_affair set complete_time = (select rec_time from edoc_exchange_recieve where ctp_affair.sub_object_id = edoc_exchange_recieve.id and ctp_affair.member_id = edoc_exchange_recieve.rec_user_id and edoc_exchange_recieve.rec_time is not null) where sub_app = 23 ";
		JDBCAgent agent = null;
		try{
			agent = new JDBCAgent(true);
			agent.execute(updateSql);
		} finally{
			if(agent != null){	
				try{
					agent.close();
				}catch (Exception e){
					LOG.error(e);
				}
			}
		}
	}
	
	private void copyExchangeSendToEdocSummary() throws BusinessException, SQLException{
		String hql  = "from EdocSendRecord where status in (0,1,3,4)";//待发送0/已发送1/已回退2/退回后生成新数据3/撤销后生成的新数据4/已发送删除5/待发送删除6
		FlipInfo fileinfo = new FlipInfo();
		fileinfo.setSize(500);
		int page = 1;
		fileinfo.setPage(page);
		int rowsCount = 0;
		List<EdocSummary> summarys = new ArrayList<EdocSummary>();
		List<EdocSendRecord> edocSendRecords= (List<EdocSendRecord> )edocUpgradeDao.getDataByHql(hql,new HashMap<String, Object>() ,fileinfo);
		rowsCount = fileinfo.getTotal();
		
		if(rowsCount == 0) {
			return;
		}
		
		Map<Long, String> summaryMap = new HashMap<Long, String>();
		do{
			LOG.info("正在操作<EdocSendRecord>第" + ((fileinfo.getPage() - 1) * fileinfo.getSize() + 1) + "条数据");
			for(EdocSendRecord record : edocSendRecords){
				EdocSummary edocSummary = new EdocSummary();
				edocSummary.setId(record.getId());
				edocSummary.setSubject(record.getSubject());
				edocSummary.setEdocType(99);
				edocSummary.setDeadline(-1L);
				edocSummary.setHasArchive(false);
				edocSummary.setGovdocType(0);
				edocSummary.setState(7);//ColConstant.flowState.oldexchange.ordinal()
				edocSummary.setDocMark(record.getDocMark());
				edocSummary.setSecretLevel(record.getSecretLevel());
				edocSummary.setUrgentLevel(record.getUrgentLevel());
				edocSummary.setCreateTime(record.getCreateTime());
				Long sourceId = record.getEdocId();
				if(sourceId != null) {
					if(!summaryMap.containsKey(sourceId)) {
						try {
							EdocSummary sourceSummary = edocUpgradeDao.getSummarySerialNo(sourceId);
							summaryMap.put(sourceSummary.getId(), sourceSummary.getSerialNo());
						} catch(Exception e) {}
					}
					if(Strings.isNotBlank(summaryMap.get(sourceId))) {
						edocSummary.setSerialNo(summaryMap.get(sourceId));
					}
				}
				try {
					V3xOrgMember mem = orgManager.getMemberById(record.getSendUserId());
					if(null != mem){
						edocSummary.setCreatePerson(mem.getName());
					}
				} catch (BusinessException e) {
				}
				edocSummary.setStartUserId(record.getSendUserId());
				edocSummary.setOrgAccountId(record.getExchangeOrgId());
				edocSummary.setStartTime(record.getSendTime());
				
				summarys.add(edocSummary);
			}
			edocUpgradeDao.saveOrUpdateAll(summarys);
			summarys.clear();
			
			if(rowsCount <= page*500){
				break;
			}
			
			fileinfo.setPage(++page);
			fileinfo.setNeedTotal(false);
			edocSendRecords= (List<EdocSendRecord> )edocUpgradeDao.getDataByHql(hql,new HashMap<String, Object>() ,fileinfo);
		}
		while(Strings.isNotEmpty(fileinfo.getData()) && fileinfo.getData().size() > 0);
		
		//更新affair结束时间
		String updateSql = "update ctp_affair set complete_time = (select send_time from edoc_exchange_send where ctp_affair.sub_object_id = edoc_exchange_send.id and edoc_exchange_send.send_time is not null) where sub_app = 22 ";
		JDBCAgent agent = null;
		try{
			agent = new JDBCAgent(true);
			agent.execute(updateSql);
		} finally{
			if(agent != null){	
				try{
					agent.close();
				}catch (Exception e){
					LOG.error(e);
				}
			}
		}
	}
	
	
	@Override
	public List<String> getTableName(String tables) throws Exception{
		List<String> tableNames = new ArrayList<String>();
		String mysql = "SELECT DISTINCT TABLE_NAME from information_schema.tables where  table_name like '"+tables+"%' and TABLE_SCHEMA=(select DATABASE())";
		String oracle = "select Table_Name from dba_tables where OWNER = (select user from dual) AND table_name like '"+tables+"%'";
		String sqlserver = "SELECT DISTINCT NAME from sys.tables where name like '"+tables+"%' AND type='U'";
		JDBCAgent agent = null;
		try{
			agent = new JDBCAgent(true);
			String dbType = JDBCAgent.getDBType().toLowerCase();
			if(dbType.indexOf("oracle") > -1 && null != oracle){
				agent.execute(oracle);
			}else if (dbType.indexOf("mysql") > -1 && null != mysql){
				agent.execute(mysql);
			}else if(dbType.indexOf("sqlserver") > -1 && null != sqlserver){
				agent.execute(sqlserver);
			}else{//兼容 达梦 等其他数据库  据说和sqlserver相同
				agent.execute(sqlserver);
			}
			ResultSet res = agent.getQueryResult();
			while(res.next()){
				tableNames.add(res.getString(1));
			}
		} finally{
			if(agent != null){	
				try{
					agent.close();
				}catch (Exception e){
					LOG.error(e);
				}
			}
		}
		
		
		return tableNames;
	}
	
	private void executeFormmainUpLevel(){
		//获取所有formmain表名
		String dbType = edocUpgradeDao.getDbType();
		List<String> tableNames = new ArrayList<String>();
		try {
			if(dbType.indexOf("oracle") > -1){
				tableNames = edocUpgradeDao.getTableName("FORMMAIN_");
			}else{
				tableNames = edocUpgradeDao.getTableName("formmain_");
			}
			
		} catch (Exception e) {
			LOG.info("查询数据库中表 ================="+e.getMessage());
		}
		//执行升级
		if(tableNames != null && tableNames.size() > 0){
			for(int i=0; i<tableNames.size(); i++){
				try {
					if(dbType.indexOf("oracle") > -1){
						edocUpgradeDao.excuteSql("alter table "+tableNames.get(i)+" add receive_unit VARCHAR2(255)");
					}else if (dbType.indexOf("mysql") > -1){
						edocUpgradeDao.excuteSql("alter table "+tableNames.get(i)+" add COLUMN receive_unit VARCHAR(255)");
					}else if(dbType.indexOf("sqlserver") > -1){
						edocUpgradeDao.excuteSql("alter table "+tableNames.get(i)+" add  receive_unit NVARCHAR(255)");
					}else{//兼容 达梦 等其他数据库  据说和sqlserver相同
						edocUpgradeDao.excuteSql("alter table "+tableNames.get(i)+" add  receive_unit NVARCHAR(255)");
					}
				} catch (Exception e) {
					LOG.error(" 升级出错 如果错误信息是字段重复，那么该错误忽略  "+e.getMessage());
				}
			}
		}
	}
	
	/**
	 * 保存老数据和新数据的对应关系
	 * @throws SQLException 
	 * @throws BusinessException 
	 */
	private void saveDataRelation(){
		Map<Long, Long> tempMap = new HashMap<Long, Long>();
		int i = 0;
		for (Long oldId : formIds.keySet()) {
			i++;
			tempMap.put(oldId, formIds.get(oldId));
			if (i % 100 == 0 || i == formIds.size()) {
				try {
					edocUpgradeDao.saveDataRelation(tempMap, "form");
				} catch (BusinessException e) {
					LOG.error("保存新老数据关系发生异常！(升级表单数据)");
					LOG.error(tempMap);
					LOG.error(e.getMessage(),e);
				} catch (SQLException e) {
					LOG.error("保存新老数据关系发生异常！(升级表单数据)");
					LOG.error(tempMap);
					LOG.error(e.getMessage(),e);
				}
				tempMap = new HashMap<Long, Long>();
			}
		}
		List<Long> tempList = new ArrayList<Long>();
		i = 0;
		for (Long oldId : presetForms) {
			i++;
			tempList.add(oldId);
			if (i % 100 == 0 || i == presetForms.size()) {
				try {
					edocUpgradeDao.saveDataRelation2(tempList, "presetForm");
				} catch (BusinessException e) {
					LOG.error("保存新老数据关系发生异常！(预置表单数据)");
					LOG.error(tempMap);
					LOG.error(e.getMessage(),e);
				} catch (SQLException e) {
					LOG.error("保存新老数据关系发生异常！(预置表单数据)");
					LOG.error(tempMap);
					LOG.error(e.getMessage(),e);
				}
				tempList = new ArrayList<Long>();
			}
		}
		
		
		i = 0;
		for (Long oldId : categories.keySet()) {
			i++;
			tempMap.put(oldId, categories.get(oldId).getId());
			if (i % 100 == 0 || i == categories.size()) {
				try {
					edocUpgradeDao.saveDataRelation(tempMap, "category");
				} catch (BusinessException e) {
					LOG.error("保存新老数据关系发生异常！(升级分类数据)");
					LOG.error(tempMap);
					LOG.error(e.getMessage(),e);
				} catch (SQLException e) {
					LOG.error("保存新老数据关系发生异常！(升级分类数据)");
					LOG.error(tempMap);
					LOG.error(e.getMessage(),e);
				}
				tempMap = new HashMap<Long, Long>();
			}
		}
		
		
		i = 0;
		for (Long newId : edocElementIds) {
			i++;
			tempList.add(newId);
			if (i % 100 == 0 || i == edocElementIds.size()) {
				try {
					edocUpgradeDao.saveDataRelation2(tempList, "edocElement");
				} catch (BusinessException e) {
					LOG.error("保存新老数据关系发生异常！(新增公文元素数据)");
					LOG.error(tempMap);
					LOG.error(e.getMessage(),e);
				} catch (SQLException e) {
					LOG.error("保存新老数据关系发生异常！(新增公文元素数据)");
					LOG.error(tempMap);
					LOG.error(e.getMessage(),e);
				}
				tempList = new ArrayList<Long>();
			}
		}
		
		i = 0;
		for (Long newId : saveRoleIds) {
			i++;
			tempList.add(newId);
			if (i % 100 == 0 || i == saveRoleIds.size()) {
				try {
					edocUpgradeDao.saveDataRelation2(tempList, "role");
				} catch (BusinessException e) {
					LOG.error("保存新老数据关系发生异常！(角色数据)");
					LOG.error(tempMap);
					LOG.error(e.getMessage(),e);
				} catch (SQLException e) {
					LOG.error("保存新老数据关系发生异常！(角色数据)");
					LOG.error(tempMap);
					LOG.error(e.getMessage(),e);
				}
				tempList = new ArrayList<Long>();
			}
		}
	}
	
	
	public Map<String, List<PrivTreeNodeBO>> getTreeNodes(String roleId,
			 String appResCategory,String isAllocated, 
			List<PrivTreeNodeBO> treeNodes4Back0, List<PrivTreeNodeBO> treeNodes4Front0)
			throws BusinessException {
		List<PrivTreeNodeBO> treeNodes4Back = new ArrayList<PrivTreeNodeBO>();
		List<PrivTreeNodeBO> treeNodes4Front = new ArrayList<PrivTreeNodeBO>();
		Map<String, List<PrivTreeNodeBO>> hasMap = new ConcurrentHashMap<String, List<PrivTreeNodeBO>>();
		// 当前登录人员拥有的所有菜单列表
		Map<Long, PrivMenuBO> menus = new HashMap<Long, PrivMenuBO>();
		// 不可修改的角色资源关系列表
		Map<Long, PrivRoleMenu> roleResUnEditable = null;
		// 如果人员ID和单位ID不为空则为查看所拥有的资源

		Long[] roleIds = new Long[1];
		if (roleId != null) {
			roleIds[0] = Long.parseLong(roleId);
		}
		
		// 获得当前角色包含的菜单
		Map<Long, PrivMenuBO> menuMap = privilegeMenuManager.getByRole(roleIds);
		if (menuMap != null && menuMap.size() != 0) {
			menus = menuMap;
		}
		// 查找不能修改的角色资源关系
		roleResUnEditable =privilegeMenuManager.findUnModifiableRoleMenuByRole(Long.parseLong(roleId));
	
		if (menus != null && menus.size() != 0) {
			PrivTreeNodeBO node = null;
			PrivRoleMenu menuUnEidtable = null;
			String menuType = null;
			List<PrivTreeNodeBO> result = null;
			String sysback = MenuTypeEnums.systemback.getValue();
			String appfront = MenuTypeEnums.applicationfront.getValue();

			// 过滤停用和不可分配的菜单
			List<PrivMenuBO> dislist = privilegeMenuManager.getConfigDisableMenu();
			if (dislist != null) {
				for (PrivMenuBO privMenuBO : dislist) {
					menus.remove(privMenuBO.getId());
				}
			}

			for (PrivMenuBO privMenuBO : menus.values()) {
				result = new ArrayList<PrivTreeNodeBO>();
				// 将菜单对象转换为树节点对象
				node = new PrivTreeNodeBO(privMenuBO, null);
				// 菜单是否可编辑
				if (roleResUnEditable != null) {
					menuUnEidtable = roleResUnEditable.get(privMenuBO.getId());
					if (menuUnEidtable != null && isAllocated != null && "true".equals(isAllocated)) {
						node.setEditKey("false");
					}
				}

				// 菜单是否可勾选
				if (privMenuBO.getExt5() != null && privMenuBO.getExt5() == 0) {
					node.setEditKey("false");
				}

				result.add(node);
				// 判断当前菜单类型
				menuType = privMenuBO.getExt1();
				if (sysback.equals(menuType)) {
					treeNodes4Back.addAll(result);
				} else if (appfront.equals(menuType)) {
					treeNodes4Front.addAll(result);
				}
			}

			// 过滤由于不可分配的菜单过滤后引起的没有子菜单且没有入口资源的菜单
			if (appResCategory != null && "1".equals(appResCategory)) {
				List<PrivTreeNodeBO> templist = new ArrayList<PrivTreeNodeBO>();
				for (int i = 0; i < treeNodes4Front.size(); i++) {
					PrivTreeNodeBO privTreeNodeBO = treeNodes4Front.get(i);
					if (privTreeNodeBO == null || privTreeNodeBO.getIdKey() == null) {
						continue;
					}
					templist.add(privTreeNodeBO);

				}
				treeNodes4Front.clear();
				treeNodes4Front.addAll(templist);
			}
		}

		for(PrivTreeNodeBO tb : treeNodes4Back){
			if(isShow(tb)){
				treeNodes4Back0.add(tb);
			}
		}
		
		for(PrivTreeNodeBO tf : treeNodes4Front){
			if(isShow(tf)){
				treeNodes4Front0.add(tf);
			}
		}
		
		hasMap.put("treeNodes4Back", treeNodes4Back0);
		hasMap.put("treeNodes4Front", treeNodes4Front0);
		return hasMap;
	}
	
	
	private boolean isShow(PrivTreeNodeBO node){
		//屏蔽掉一些菜单
		Long menuId = node.getMenu().getId();
        //A6-S屏蔽的菜单
        Integer productId = SystemProperties.getInstance().getIntegerProperty("system.ProductId");
        if (productId != null && productId.intValue() == ProductEditionEnum.a6s.ordinal()) {
        	if(menuId.equals(-5229428449906440478L)
             || menuId.equals(-5229428449906440478L)//系统邮箱设置
             || menuId.equals(6692870513281941850L)//HR管理-组织机构设置
             || menuId.equals(7464083475998789371L)//HR管理-员工档案管理
             || menuId.equals(-2933570987085508976L)//HR管理-统计分析
             || menuId.equals(-6400113376465108785L)){//HR管理-信息项设置
        		return false;
        	}
        }

        if(ProductEditionEnum.isU8OEM()){
        	//屏蔽集成平台配置
        	if(menuId.equals(-1726650702758329325L)//系统注册
        	 || menuId.equals(-1461605504161199936L)//用户管理
        	 || menuId.equals(1619292301944899707L)//消息配置
        	 || menuId.equals(-4628815616839168895L)//待办配置
        	 || menuId.equals(-2176243471020961962L)//门户配置
        		//屏蔽移动office授权
        	 || menuId.equals(7811846044579348610L)//移动office授权
        		//屏蔽应用中心设置
        	 || menuId.equals(-329292124654313901L)){//云应用中心设置
        		return false;
        	}
        }
        
        return true;
	}
	
	private void insterEdocElements() {
		LOG.info("添加公文元素开始...");
		if (ProductEditionEnum.getCurrentProductEditionEnum().getValue().equals(ProductEditionEnum.enterprise.getValue())
			|| ProductEditionEnum.getCurrentProductEditionEnum().getValue().equals(ProductEditionEnum.a6.getValue()) ) {
			return;
		}
		for (V3xOrgAccount v3xOrgAccount : units) {
			if (v3xOrgAccount.getId().equals(-1730833917365171641L)) {
				continue;
			}
			try {
				List<String> sqls = new ArrayList<String>();
				Long id1 = UUIDLong.longUUID();
				Long id2 = UUIDLong.longUUID();
				Long id3 = UUIDLong.longUUID();
				Long id4 = UUIDLong.longUUID();
				sqls.add("INSERT INTO EDOC_ELEMENT(ID,ELEMENT_ID,NAME,FIELD_NAME,TYPE,METADATA_ID,IS_SYSTEM,STATUS,DOMAIN_ID,INPUT_MODE) VALUES ('"+id1+"', '371', 'edoc.element.receiveunit', 'receive_unit', '0', null, '1', '1','"+v3xOrgAccount.getId()+"','')");
				sqls.add("INSERT INTO EDOC_ELEMENT(ID,ELEMENT_ID,NAME,FIELD_NAME,TYPE,METADATA_ID,IS_SYSTEM,STATUS,DOMAIN_ID,INPUT_MODE) VALUES ('"+id2+"', '372', 'edoc.element.signmark', 'sign_mark', '0', null, '1', '1','"+v3xOrgAccount.getId()+"','')");
				sqls.add("INSERT INTO edoc_element(ID,ELEMENT_ID,NAME,FIELD_NAME,TYPE,METADATA_ID,IS_SYSTEM,STATUS,DOMAIN_ID,INPUT_MODE) VALUES ('"+id3+"', '374', 'edoc.element.baopiyijian', 'baopiyijian', 0, NULL, 1, 1, '"+v3xOrgAccount.getId()+"', '')");
				sqls.add("INSERT INTO EDOC_ELEMENT(ID,ELEMENT_ID,NAME,FIELD_NAME,TYPE,METADATA_ID,IS_SYSTEM,STATUS,DOMAIN_ID,INPUT_MODE) VALUES ('"+id4+"', '375', 'edoc.element.signperson', 'sign_person', '0', null, '1', '1','"+v3xOrgAccount.getId()+"','')");
				edocElementIds.add(id1);
				edocElementIds.add(id2);
				edocElementIds.add(id3);
				edocElementIds.add(id4);
				edocUpgradeDao.insterEdocElements(sqls);
			} catch (BusinessException e) {
				LOG.error("添加公文元素出错！单位:" + v3xOrgAccount.getName());
				LOG.error(e.getMessage(),e);
			} catch (SQLException e) {
				LOG.error("添加公文元素出错！单位:" + v3xOrgAccount.getName());
				LOG.error(e.getMessage(),e);
			}
		}
		LOG.info("添加公文元素结束...");
	}
	
	private void presetgovdocForms() {
		LOG.info("预置公文单开始...");
		for (V3xOrgAccount v3xOrgAccount : units) {
			if (v3xOrgAccount.getId().equals(-1730833917365171641L)) {
				continue;
			}
			govdocForm(v3xOrgAccount);
		}
		LOG.info("预置公文单结束...");
	}
	
	private Map<String, Object> govdocForm(File xsnFile,Long fileId,V3xOrgAccount v3xOrgAccount, String state, long categoryId, String rootPath, int formType, long formId) throws BusinessException {
		Map<String, Object> result = new HashMap<String, Object>();
        result.put("success", false);
        FormBean fb = null;
		if (fileId != -1L) {
			File file = xsnFile;
			try {
				file = this.fileManager.getFile(fileId, DateUtil.currentDate());
				if (file == null) {
					file = xsnFile;
				}else {
					file = this.fileManager.decryptionFile(file);
				}
			} catch (Exception e) {
				file = xsnFile;
			}
			V3XFile v3 = this.fileManager.getV3XFile(fileId);
			try {
				InfoPathObject xsf = formApi4Cap3.parseXSN(file);
				//infopath的各种校验
				String check = validateInfopath(xsf,formType,formId);
				if (!"".equals(check)) {
                    result.put("msg", check);
                    return result;
				}
				// 这里主要是进行表格控件查核,如果查核有问题,则会抛出BusinessException,后面会接住此Exception,并return
				for (int i = 0; i < xsf.getViewList().size(); i++) {
					InfoPath_xsl xsl = xsf.getViewList().get(i);
					xsl.covertContent(null);
				}
				if (formId == -1L) {
					fb = formApi4Cap3.getFormBeanForUpgrade2(xsf,formType,v3xOrgAccount.getId());
					// OA-1063 上传infopath表名和实际的不一致
					String fileName = v3.getFilename();
					fb.setFormName(fileName.substring(0, fileName.lastIndexOf(".")));
					fb.setFormType(formType);// 类型
					if(fb.getFormType() == Enums.FormType.govDocSendForm.getKey()||fb.getFormType() == Enums.FormType.govDocSignForm.getKey()
						|| fb.getFormType() == Enums.FormType.govDocReceiveForm.getKey()||fb.getFormType() == Enums.FormType.govDocExchangeForm.getKey()){
						fb.setGovDocFormType(fb.getFormType());
						fb.setFormType(Enums.FormType.processesForm.getKey());
			        }
					// 增加此表单的源xsn文件关系
					List<FormResource> resourceList = (List<FormResource>) fb.getExtraAttr("infoPathResource");
					FormResource fr = new FormResource();
					fr.setId(fileId);
					fr.setFormId(fb.getId());
					fr.setResourceName("name.xsn");
					fr.setPropertyType(FormResourcePropertyTypeEnum.InfoPath.getKey());
					fr.setPropertyName("name.xsn");
					fr.setContent("name.xsn");
					resourceList.add(fr);
					fb.putExtraAttr("infoPathResource", resourceList);
					fb.setCreatorId(v3xOrgAccount.getId());
			        fb.setOwnerId(v3xOrgAccount.getId());
				}
			} catch (Exception e) {
			    LOG.error(e.getMessage(), e);
                result.put("msg", e.getMessage());
                return result;
			}
		} else {
            result.put("msg", ResourceUtil.getString("form.fielddesign.cantfindfileid"));
            return result;
		}

		//预置交换单应用绑定开始
		if(formType==FormType.govDocExchangeForm.getKey()){
			String wfProcessTempleteXML0 = System.currentTimeMillis()+"0";
			String wfProcessTempleteXML1 = System.currentTimeMillis()+"1";
			String wfProcessTempleteXML2 = System.currentTimeMillis()+"2";
			String wfProcessTempleteXML4 = System.currentTimeMillis()+"4";
			String wfProcessTempleteXML5 = System.currentTimeMillis()+"5";
			List<FormAuthViewBean> operationList = fb.getFormViewList().get(0).getAllOperations();
			Long tianxieId = 0L;
			Long shenpiId = 0L;
			String fieldName = "";
			for (FormAuthViewBean operation : operationList) {
        		if("add".equals(operation.getType())){
        			tianxieId = operation.getId();
        		}else if("update".equals(operation.getType())){
        			shenpiId = operation.getId();
        		}
            }
			
			List<FormFieldBean> formFieldBeans = fb.getAllFieldBeans();
			for (FormFieldBean formFieldBean : formFieldBeans) {
				if (ElementUtil.GovDocElement.receive_unit.getCode().equals(formFieldBean.getMappingField())) {
					fieldName = formFieldBean.getName();
					break;
				}
			}
			
			Map<String, String> map = new HashMap<String, String>();
			map.put("id", "");
			map.put("subject", "单位签收流程");
			map.put("govdocContent", "");
			map.put("govdocContentType", " }");
			map.put("govdocBodyType", "OfficeWord");
			map.put("isFlowCopy", "0");
			map.put("process_id", "");
			map.put("taohongTemplete", "-1");
			map.put("process_desc_by", "xml");
			map.put("process_subsetting", "{}");
			map.put("process_rulecontent", "");
			map.put("workflow_newflow_input", "");
			map.put("workflow_node_peoples_input", "");
			map.put("workflow_node_condition_input", "");
			map.put("process_xml_clone", "");
			map.put("process_xml_clone2flowCopy", "");
			map.put("process_event", "{}");
			map.put("process_info", "接收单位单位公文收文员(签收)、上节点部门公文送文员(分办)");
			map.put("colSubject", "");
			map.put("contentType", "41");
			map.put("contentRed", "-1");
			map.put("uploadFile", "");
			map.put("markBind", "");
			map.put("importantLevel", "1");
			map.put("projectId", "");
			map.put("cycleState", "");
			map.put("cycleSender", "");
			map.put("cycleSender_txt", "");
			map.put("cycleStartDate", "");
			map.put("cycleEndDate", "");
			map.put("cycleType", "");
			map.put("cycleMonth", "");
			map.put("cycleOrder", "");
			map.put("cycleDay", "");
			map.put("cycleWeek", "");
			map.put("cycleHour", "");
			map.put("oldDeadlineValue", "");
			map.put("deadline", "0");
			map.put("standardDuration", "0");
			map.put("advanceremind", "0");
			map.put("archiveId", "");
			map.put("archiveName", "");
			map.put("archiveFieldName", "");
			map.put("archiveIsCreate", "true");
			map.put("canTrackWorkFlow", "0");
			map.put("canForward", "");
			map.put("canModify", "1");
			map.put("canArchive", "1");
			map.put("canEditAttachment", "");
			map.put("canMergeDeal", "");
			map.put("templateNumber", "");
			map.put("showSupervisors", "");
			map.put("canSupervise", "1");
			map.put("authRelation_txt", "");
			map.put("authRelation", "");
			map.put("dep_auth_txt", v3xOrgAccount.getName());
			map.put("dep_auth", "Account|"+v3xOrgAccount.getId());
			map.put("contentFileId", UUID.randomUUID().getLeastSignificantBits()+"");
			map.put("view_"+fb.getFormViewList().get(0).getId(), "null");
			map.put("auth_"+fb.getFormViewList().get(0).getId(), tianxieId+"");
			map.put("process_xml", "<ps><p s=\"false\" u=\"\" y=\"0\" x=\"0\" " +
					"t=\"p\" d=\"\" n=\"\" i=\"\"><n a=\"1\" b=\"normal\" c=\"false\" " +
					"l=\"1000\" e=\"0\" f=\"\" g=\"\" h=\"\" y=\"50\" x=\"260\" t=\"6\" " +
					"d=\"\" n=\"上节点部门公文送文员\" " +
					"i=\""+wfProcessTempleteXML4+"\">" +
					"<a a=\"\" b=\""+v3xOrgAccount.getId()+"\" c=\"1\" " +
					"d=\"上节点部门公文送文员\" e=\"\" f=\"NodeUserDepartmentGovdocSend\" " +
					"g=\"Node\" h=\"false\" i=\"false\" j=\"false\" k=\"roleadmin\" />" +
					"<s sa=\"0\" a=\"\" b=\"0\" c=\"1\" z=\"\" " +
					"r=\""+shenpiId+"\" " +
					"e=\""+fb.getFormViewList().get(0).getId()+"\" " +
					"f=\""+fb.getId()+"\" g=\"0\" na=\"0\" w=\"1\" " +
					"v=\"1\" sid=\"\" qid=\"\" h=\"-1\" u=\"-1\" tm=\"1\" j=\"single\" k=\"0\" " +
					"l=\"0\" m=\"false\" s=\"success\" o=\"null\" p=\"null\" q=\"0\" d=\"\" " +
					"t=\"\" n=\"分办\" i=\"fenban\" /></n><n f=\"\" g=\"\" h=\"\" y=\"50\" x=\"50\" " +
					"t=\"8\" d=\"\" n=\"发起者\" i=\"start\"><a a=\"\" b=\"\" c=\"1\" d=\"\" e=\"\" " +
					"f=\"\" g=\"\" h=\"false\" i=\"false\" j=\"false\" k=\"\" /><s sa=\"0\" a=\"\" b=\"0\" " +
					"c=\"1\" z=\"\" r=\""+tianxieId+"\" " +
					"e=\""+fb.getFormViewList().get(0).getId()+"\" " +
					"f=\""+fb.getId()+"\" g=\"0\" na=\"-1\" " +
					"w=\"-1\" v=\"-1\" sid=\"\" qid=\"\" h=\"-1\" u=\"-1\" tm=\"1\" j=\"\" " +
					"k=\"\" l=\"\" m=\"false\" s=\"success\" o=\"\" p=\"\" q=\"\" d=\"\" t=\"\" " +
					"n=\"签收\" i=\"qianshou\" /></n><n f=\"\" g=\"\" h=\"\" y=\"50\" x=\"365\" t=\"4\" " +
					"d=\"\" n=\"end\" i=\"end\"><a a=\"\" b=\"\" c=\"1\" d=\"\" e=\"\" f=\"\" g=\"\" " +
					"h=\"false\" i=\"false\" j=\"false\" k=\"\" /><s sa=\"0\" a=\"\" b=\"0\" c=\"1\" " +
					"z=\"\" r=\""+shenpiId+"\" " +
					"e=\""+fb.getFormViewList().get(0).getId()+"\" " +
					"f=\""+fb.getId()+"\" g=\"0\" na=\"-1\" w=\"-1\" " +
					"v=\"-1\" sid=\"\" qid=\"\" h=\"-1\" u=\"-1\" tm=\"1\" j=\"\" k=\"\" " +
					"l=\"\" m=\"false\" s=\"success\" o=\"\" p=\"\" q=\"\" d=\"\" t=\"\" n=\"签收\" " +
					"i=\"qianshou\" /></n><n a=\"1\" b=\"normal\" c=\"false\" l=\"1000\" e=\"0\" " +
					"f=\"\" g=\"\" h=\"\" y=\"50\" x=\"155\" t=\"6\" d=\"\" n=\"接收单位单位公文收文员\" " +
					"i=\""+wfProcessTempleteXML1+"\"><a a=\"\" " +
					"b=\""+v3xOrgAccount.getId()+"\" c=\"1\" d=\"接收单位单位公文收文员\" e=\"\" " +
					"f=\"AccountAndDepartment@"+fieldName+"#接收单位#"+orgManager.getRoleByName(OrgConstants.Role_NAME.AccountGovdocRec.name(), v3xOrgAccount.getId()).getId()+
					"\" g=\"FormField\" h=\"false\" i=\"false\" j=\"false\" k=\"roleadmin\" />" +
					"<s sa=\"0\" a=\"\" b=\"0\" c=\"1\" z=\"\" " +
					"r=\""+shenpiId+"\" " +
					"e=\""+fb.getFormViewList().get(0).getId()+"\" " +
					"f=\""+fb.getId()+"\" g=\"0\" na=\"-1\" " +
					"w=\"-1\" v=\"-1\" sid=\"\" qid=\"\" h=\"-1\" u=\"-1\" tm=\"1\" j=\"competition\" " +
					"k=\"\" l=\"\" m=\"false\" s=\"success\" o=\"\" p=\"\" q=\"\" d=\"\" t=\"\" " +
					"n=\"签收\" i=\"qianshou\" /></n><l a=\"\" b=\"\" c=\"\" m=\"\" e=\"\" " +
					"h=\"3\" o=\"0\" j=\"end\" " +
					"k=\""+wfProcessTempleteXML4+"\" " +
					"t=\"11\" d=\"\" n=\"\" " +
					"i=\""+wfProcessTempleteXML0+"\" />" +
					"<l a=\"\" b=\"\" c=\"\" m=\"\" e=\"\" h=\"3\" o=\"0\" " +
					"j=\""+wfProcessTempleteXML4+"\" " +
					"k=\""+wfProcessTempleteXML1+"\" " +
					"t=\"11\" d=\"\" n=\"\" i=\""+wfProcessTempleteXML5+"\" />" +
					"<l a=\"\" b=\"\" c=\"\" m=\"\" e=\"\" h=\"3\" o=\"0\" " +
					"j=\""+wfProcessTempleteXML1+"\" " +
					"k=\"start\" t=\"11\" d=\"\" n=\"\" " +
					"i=\""+wfProcessTempleteXML2+"\" /></p></ps>");

			map.put("groupNewAccountId", v3xOrgAccount.getId()+"");
			map.put("groupNewMemberId", v3xOrgAccount.getId()+"");
			map.put("groupNewBatchId", UUID.randomUUID().getLeastSignificantBits()+"");
			
			formApi4Cap3.saveOrUpdateFlowTemplate4Upgrade(map, new ArrayList<Map<String,String>>(),v3xOrgAccount,fb);
		}
		//预置交换单应用绑定结束
		
		fb.setCategoryId(categoryId);
		fb.setOwnerId(v3xOrgAccount.getId());
		GovDocUtil.saveGovdocExtendAndSort(fb, govdocFormExtendManager, govdocFormOpinionSortManager);
		fb.setState(Integer.parseInt(state));
		
		fb.putExtraAttr("groupNewAccountId", v3xOrgAccount.getId()+"");

		List<FormViewBean> viewList = fb.getFormViewList();
        for (FormViewBean view : viewList) {
            List<FormAuthViewBean> operationList = view.getAllOperations();
            for (FormAuthViewBean operation : operationList) {
            	List<FormAuthViewFieldBean> authViewFieldBeans = operation.getFormAuthorizationFieldList();
            	for (FormAuthViewFieldBean formAuthViewFieldBean : authViewFieldBeans) {
            		if("add".equals(operation.getType())){
            			formAuthViewFieldBean.setAccess(FieldAccessType.edit.getKey());
            		}else{
            			formAuthViewFieldBean.setAccess(FieldAccessType.browse.getKey());
            		}
				}
            }
            
        }
		
		FormOwner formOwner = formApi4Cap3.getFormOwner(fb.getId());
		//屏蔽--许才兵
//		deeDesignManager.setDeeTask4FormField(fb);
		try {
			if (formApi4Cap3.isNewForm(fb.getId())) {
				if (fb.needCheckTotalNum()) {
					BizValidateResultVO resultVO = formApi4Cap3.validateBiz(BizOperationEnum.create_form_save);
					if (!resultVO.isSuccess()) {
						throw new BizException(resultVO.getCode());
					}
				}
				fb.setSourceStr(formApi4Cap3.getEncodeString(formApi4Cap3.getCreateSourceType(BizOperationEnum.create_form_save), fb.getId()));
			}
			if (fb.getGovDocFormType() == Enums.FormType.govDocSendForm.ordinal()) {
				String item = EnumNameEnum.edoc_send_permission_policy.name();
				String newItem = EnumNameEnum.edoc_new_send_permission_policy.name();
				fb.setFormPermissionConfig(this.getFormPermissionConfig(item, newItem, v3xOrgAccount.getId(), fb,null));
			}else if (fb.getGovDocFormType() == Enums.FormType.govDocReceiveForm.ordinal()) {
				String item = EnumNameEnum.edoc_rec_permission_policy.name();
				String newItem = EnumNameEnum.edoc_new_rec_permission_policy.name();
				fb.setFormPermissionConfig(this.getFormPermissionConfig(item, newItem, v3xOrgAccount.getId(), fb,null));
			}
			presetForms.add(fb.getId());
			formApi4Cap3.saveOrUpdateFormBean(fb);
		} catch (BizException be) {
			result.put("msg", ResourceUtil.getString(be.getErrorCode()));
            return result;
		} catch (BusinessException e) {
			if(e.getCause()!=null && e.getCause().getMessage().contains("65535")){
				//OA-60624 MYSQL字段超出范围 报错控制
				result.put("msg", ResourceUtil.getString("form.baseinfo.totalLength.formDesignSave.error"));
	            return result;
			}else{
				throw e;
			}
		} catch (Exception e) {
			result.put("msg", "单位"+ v3xOrgAccount.getName() + "预置文单《"+fb.getFormName()+"》发生异常！");
			LOG.error(e.getMessage(),e);
            return result;
		}
		
		result.put("success", true);
        
		return result;
	}
	
	
	/**
	 * 预置发文单，收文单，签收单数据
	 * @param v3xOrgAccount
	 */
	private void govdocForm(V3xOrgAccount v3xOrgAccount) {
		long userId = v3xOrgAccount.getId();
		String applicationFolder = SystemEnvironment.getApplicationFolder();
		String baseFolder = SystemEnvironment.getBaseFolder();
		
		try {
			fileManager.getFolder(new Date(), true);
		} catch (BusinessException e1) {
			LOG.error(e1);
		}
		
		try {
			Map<String, Object> result = new HashMap<String, Object>();
			LOG.info("初始化单位："+v3xOrgAccount.getName()+"发文单数据开始...");
			File fileInGovDocSendForm = new File(applicationFolder+File.separator+"apps_res"+File.separator+"form"+File.separator+"file"+File.separator+"govDocSendForm");
			Date dateGovDocSendForm = new Date();
			V3XFile v3xFileGovDocSendForm = new V3XFile();
			v3xFileGovDocSendForm.setIdIfNew();
			v3xFileGovDocSendForm.setCategory(1);
			v3xFileGovDocSendForm.setType(0);
			v3xFileGovDocSendForm.setFilename("发文单.xsn");
			v3xFileGovDocSendForm.setMimeType("application/octet-stream");
			v3xFileGovDocSendForm.setCreateDate(dateGovDocSendForm);
			v3xFileGovDocSendForm.setCreateMember(userId);
			v3xFileGovDocSendForm.setSize(fileInGovDocSendForm.length());
			v3xFileGovDocSendForm.setDescription("");
			v3xFileGovDocSendForm.setUpdateDate(dateGovDocSendForm);
			v3xFileGovDocSendForm.setAccountId(v3xOrgAccount.getId());
			v3xFileDAO.save(v3xFileGovDocSendForm);
			File fileOutGovDocSendForm = new File(baseFolder+File.separator+"upload"+File.separator+
					new SimpleDateFormat("yyyy").format(dateGovDocSendForm)+
					File.separator+new SimpleDateFormat("MM").format(dateGovDocSendForm)+
					File.separator+new SimpleDateFormat("dd").format(dateGovDocSendForm)+
					File.separator+v3xFileGovDocSendForm.getId());

			try {
				FileUtil.copyFile(fileInGovDocSendForm, fileOutGovDocSendForm);
			} catch (Exception e) {
				fileOutGovDocSendForm = fileInGovDocSendForm;
			}
			result = govdocForm(fileInGovDocSendForm,v3xFileGovDocSendForm.getId(), v3xOrgAccount, "2", ModuleType.govdocSend.getKey(), SystemEnvironment.getContextPath(), FormType.govDocSendForm.getKey(), -1L);
			if((Boolean) result.get("success")){
				LOG.info("初始化单位："+v3xOrgAccount.getName()+"发文单数据成功...");
			}else{
				LOG.error("初始化单位："+v3xOrgAccount.getName()+"发文单数据失败..."+result.get("msg"));
			}
			LOG.info("初始化单位："+v3xOrgAccount.getName()+"发文单数据结束...");
			LOG.info("初始化单位："+v3xOrgAccount.getName()+"收文单数据开始...");
			File fileInGovDocReceiveForm = new File(applicationFolder+File.separator+"apps_res"+File.separator+"form"+File.separator+"file"+File.separator+"govDocReceiveForm");
			Date dateGovDocReceiveForm = new Date();
			V3XFile v3xFileGovDocReceiveForm = new V3XFile();
			v3xFileGovDocReceiveForm.setIdIfNew();
			v3xFileGovDocReceiveForm.setCategory(1);
			v3xFileGovDocReceiveForm.setType(0);
			v3xFileGovDocReceiveForm.setFilename("收文单.xsn");
			v3xFileGovDocReceiveForm.setMimeType("application/octet-stream");
			v3xFileGovDocReceiveForm.setCreateDate(dateGovDocReceiveForm);
			v3xFileGovDocReceiveForm.setCreateMember(userId);
			v3xFileGovDocReceiveForm.setSize(fileInGovDocReceiveForm.length());
			v3xFileGovDocReceiveForm.setDescription("");
			v3xFileGovDocReceiveForm.setUpdateDate(dateGovDocReceiveForm);
			v3xFileGovDocReceiveForm.setAccountId(v3xOrgAccount.getId());
			v3xFileDAO.save(v3xFileGovDocReceiveForm);
			File fileOutGovDocReceiveForm = new File(baseFolder+File.separator+"upload"+File.separator+
					new SimpleDateFormat("yyyy").format(dateGovDocReceiveForm)+
					File.separator+new SimpleDateFormat("MM").format(dateGovDocReceiveForm)+
					File.separator+new SimpleDateFormat("dd").format(dateGovDocReceiveForm)+
					File.separator+v3xFileGovDocReceiveForm.getId());
			try {
				FileUtil.copyFile(fileInGovDocReceiveForm, fileOutGovDocReceiveForm);
			} catch (Exception e) {
				fileOutGovDocReceiveForm = fileInGovDocReceiveForm;
			}
			result = govdocForm(fileInGovDocReceiveForm,v3xFileGovDocReceiveForm.getId(), v3xOrgAccount, "2", ModuleType.govdocRec.getKey(), SystemEnvironment.getContextPath(), FormType.govDocReceiveForm.getKey(), -1L);
			if((Boolean) result.get("success")){
				LOG.info("初始化单位："+v3xOrgAccount.getName()+"收文单数据成功...");
			}else{
				LOG.error("初始化单位："+v3xOrgAccount.getName()+"收文单数据失败..."+result.get("msg"));
			}
			LOG.info("初始化单位："+v3xOrgAccount.getName()+"收文单数据结束...");
			LOG.info("初始化单位："+v3xOrgAccount.getName()+"签报单数据开始...");
			File fileInGovDocSignForm = new File(applicationFolder+File.separator+"apps_res"+File.separator+"form"+File.separator+"file"+File.separator+"govDocSignForm");
			Date dateGovDocSignForm = new Date();
			V3XFile v3xFileGovDocSignForm = new V3XFile();
			v3xFileGovDocSignForm.setIdIfNew();
			v3xFileGovDocSignForm.setCategory(1);
			v3xFileGovDocSignForm.setType(0);
			v3xFileGovDocSignForm.setFilename("签报单.xsn");
			v3xFileGovDocSignForm.setMimeType("application/octet-stream");
			v3xFileGovDocSignForm.setCreateDate(dateGovDocSignForm);
			v3xFileGovDocSignForm.setCreateMember(userId);
			v3xFileGovDocSignForm.setSize(fileInGovDocSignForm.length());
			v3xFileGovDocSignForm.setDescription("");
			v3xFileGovDocSignForm.setUpdateDate(dateGovDocSignForm);
			v3xFileGovDocSignForm.setAccountId(v3xOrgAccount.getId());
			v3xFileDAO.save(v3xFileGovDocSignForm);
			File fileOutGovDocSignForm = new File(baseFolder+File.separator+"upload"+File.separator+
					new SimpleDateFormat("yyyy").format(dateGovDocSignForm)+
					File.separator+new SimpleDateFormat("MM").format(dateGovDocSignForm)+
					File.separator+new SimpleDateFormat("dd").format(dateGovDocSignForm)+
					File.separator+v3xFileGovDocSignForm.getId());
			try {
				FileUtil.copyFile(fileInGovDocSignForm, fileOutGovDocSignForm);
			} catch (Exception e) {
				fileOutGovDocSignForm = fileInGovDocSignForm;
			}
			result = govdocForm(fileInGovDocSignForm,v3xFileGovDocSignForm.getId(), v3xOrgAccount, "2", ModuleType.govdocSign.getKey(), SystemEnvironment.getContextPath(), FormType.govDocSignForm.getKey(), -1L);
			if((Boolean) result.get("success")){
				LOG.info("初始化单位："+v3xOrgAccount.getName()+"签报单数据成功...");
			}else{
				LOG.error("初始化单位："+v3xOrgAccount.getName()+"签报单数据失败..."+result.get("msg"));
			}
			LOG.info("初始化单位："+v3xOrgAccount.getName()+"签报单数据结束...");
			LOG.info("初始化单位："+v3xOrgAccount.getName()+"交换单数据开始...");
			File fileInGovDocExchangeForm = new File(applicationFolder+File.separator+"apps_res"+File.separator+"form"+File.separator+"file"+File.separator+"govDocExchangeForm");
			Date dateGovDocExchangeForm = new Date();
			V3XFile v3xFileGovDocExchangeForm = new V3XFile();
			v3xFileGovDocExchangeForm.setIdIfNew();
			v3xFileGovDocExchangeForm.setCategory(1);
			v3xFileGovDocExchangeForm.setType(0);
			v3xFileGovDocExchangeForm.setFilename("签收单.xsn");
			v3xFileGovDocExchangeForm.setMimeType("application/octet-stream");
			v3xFileGovDocExchangeForm.setCreateDate(dateGovDocExchangeForm);
			v3xFileGovDocExchangeForm.setCreateMember(userId);
			v3xFileGovDocExchangeForm.setSize(fileInGovDocExchangeForm.length());
			v3xFileGovDocExchangeForm.setDescription("");
			v3xFileGovDocExchangeForm.setUpdateDate(dateGovDocExchangeForm);
			v3xFileGovDocExchangeForm.setAccountId(v3xOrgAccount.getId());
			v3xFileDAO.save(v3xFileGovDocExchangeForm);
			File fileOutGovDocExchangeForm = new File(baseFolder+File.separator+"upload"+File.separator+
					new SimpleDateFormat("yyyy").format(dateGovDocExchangeForm)+
					File.separator+new SimpleDateFormat("MM").format(dateGovDocExchangeForm)+
					File.separator+new SimpleDateFormat("dd").format(dateGovDocExchangeForm)+
					File.separator+v3xFileGovDocExchangeForm.getId());
			try {
				FileUtil.copyFile(fileInGovDocExchangeForm, fileOutGovDocExchangeForm);
			} catch (Exception e) {
				fileOutGovDocExchangeForm = fileInGovDocExchangeForm;
			}
			result = govdocForm(fileInGovDocExchangeForm,v3xFileGovDocExchangeForm.getId(), v3xOrgAccount, "2", ModuleType.govdocExchange.getKey(), SystemEnvironment.getContextPath(), FormType.govDocExchangeForm.getKey(), -1L);
			if((Boolean) result.get("success")){
				LOG.info("初始化单位："+v3xOrgAccount.getName()+"交换单数据成功...");
			}else{
				LOG.error("初始化单位："+v3xOrgAccount.getName()+"交换单数据失败..."+result.get("msg"));
			}
			LOG.info("初始化单位："+v3xOrgAccount.getName()+"交换单数据结束...");
		} catch (BusinessException e) {
			LOG.error("创建单位:"+v3xOrgAccount.getName()+"初始化文单出错！",e);
		}
	}
	
	private void createFormByEdocForm() throws BusinessException {
		LOG.info("公文单升级开始...");

		//升级所有的发文种类
		List<CtpTemplateCategory> templateCategories = edocUpgradeDao.getAllCtpTemplateCategory();
		
		for (CtpTemplateCategory templateCategory : templateCategories) {
			categories.put(templateCategory.getId(), templateCategory);
		}
		List<EdocForm> edocForms = edocUpgradeDao.getAllEdocForm();
		V3XFile v3xFileGovDocSendForm = null;
		Long fileId = null;
		Long userId = null;
		Long categoryId = null;
		Integer formType = null;
		String defaultFileId = "";
		String filePath = SystemEnvironment.getApplicationFolder()+File.separator+"apps_res"+File.separator+"edoc"+File.separator+"file"+File.separator+"form"+File.separator;
		boolean isDefaultForm = false;
		for (EdocForm edocForm : edocForms) {
			int useFlag = 1;
			try {
				LOG.info("edocForm.name="+edocForm.getName()+",id="+edocForm.getId()
				+",size="+ (null != edocForm.getEdocFormExtendInfo() ? edocForm.getEdocFormExtendInfo().size() : "0"));
				for (EdocFormExtendInfo info : edocForm.getEdocFormExtendInfo()) {
					LOG.info(edocForm.getDomainId()+"**"+info.getAccountId()+"**"+edocForm.getDomainId().equals(info.getAccountId())+"**"+info.getStatus());
					if (edocForm.getDomainId().equals(info.getAccountId())) {
						useFlag = info.getStatus();
						isDefaultForm = info.getIsDefault();
						break;
					}
				}
			} catch (Exception e2) {
				LOG.info("use previous form data");
			}
			if (formIds.get(edocForm.getId()) != null) {
				continue;
			}
			if (edocForm.getDomainId().equals(1L)) {
				continue;
			}
			String item = "";
			String newItem = "";
			LOG.info("edocForm.getType().intValue()="+edocForm.getType().intValue());
			if (edocForm.getType().intValue() == 0) {
				categoryId = Long.valueOf(ModuleType.govdocSend.getKey());
				formType = FormType.govDocSendForm.getKey();
				item = EnumNameEnum.edoc_send_permission_policy.name();
				newItem = EnumNameEnum.edoc_new_send_permission_policy.name();
				defaultFileId = "6071519916662539448";
			}else if (edocForm.getType().intValue() == 1) {
				categoryId = Long.valueOf(ModuleType.govdocRec.getKey());
				formType = FormType.govDocReceiveForm.getKey();
				item = EnumNameEnum.edoc_rec_permission_policy.name();
				newItem = EnumNameEnum.edoc_new_rec_permission_policy.name();
				defaultFileId = "-2921628185995099164";
			}else if (edocForm.getType().intValue() == 2) {
				categoryId = Long.valueOf(ModuleType.govdocSign.getKey());
				formType = FormType.govDocSignForm.getKey();
				item = EnumNameEnum.edoc_qianbao_permission_policy.name();
				newItem = EnumNameEnum.edoc_new_qianbao_permission_policy.name();
				defaultFileId = "-1766191165740134579";
			}
			LOG.info("formType="+formType);
			userId = edocForm.getCreateUserId();
			
			/**
			 * OA-168206 老文单升级不成功 createuserid 为-8060678314709529794 在org_member表里面找不到这里统一改成单位管理员ID
			 * a6设置成系统管理员 A8设置成单位管理员
			 */
			if(userId != null && userId.equals(-8060678314709529794L)){
				if(ProductEditionEnum.getCurrentProductEditionEnum().getValue().equals(ProductEditionEnum.a6.getValue())
				 || ProductEditionEnum.getCurrentProductEditionEnum().getValue().equals(ProductEditionEnum.a6s.getValue())){
					edocForm.setCreateUserId(-7273032013234748168L);
					userId = -7273032013234748168L;
				}else{
					edocForm.setCreateUserId(-884316703172445L);
					userId = -884316703172445L;
				}
			}
			
			V3xOrgMember member = orgManager.getMemberById(userId);
            if (member == null) {//5725175934914479521集团管理员
            	member = orgManager.getAdministrator(edocForm.getDomainId());
            	if (member == null) {
            		LOG.error("公文单《" + edocForm.getName()+"》跳过升级，原因：没有创建者ID" + edocForm.getId());
            		continue;
				}
            	userId = member.getId();
            	edocForm.setCreateUserId(userId);
            }
            File xsnFile = null;
			V3XFile v3xFile = v3xFileDAO.get(edocForm.getFileId());
			if (v3xFile != null) {
				xsnFile = fileManager.getFile(edocForm.getFileId(), v3xFile.getCreateDate());
			}
			if (xsnFile == null && edocForm.getIsSystem()) {
				xsnFile = new File(filePath+edocForm.getFileId());
				if(xsnFile == null || !xsnFile.exists()){
					xsnFile = new File(filePath+defaultFileId);
				}
			}
			if(xsnFile == null || !xsnFile.exists()){
				LOG.error("表单《" + edocForm.getName()+"》xsn文件未找到，请检查upload文件夹！");
				continue;
			}
			v3xFileGovDocSendForm = new V3XFile();
			v3xFileGovDocSendForm.setIdIfNew();
			v3xFileGovDocSendForm.setCategory(1);
			v3xFileGovDocSendForm.setType(0);
			v3xFileGovDocSendForm.setFilename(edocForm.getName() + ".xsn");
			v3xFileGovDocSendForm.setMimeType("application/octet-stream");
			v3xFileGovDocSendForm.setCreateDate(new Date());
			v3xFileGovDocSendForm.setCreateMember(edocForm.getCreateUserId());
			v3xFileGovDocSendForm.setSize(xsnFile.length());
			v3xFileGovDocSendForm.setDescription("");
			v3xFileGovDocSendForm.setUpdateDate(edocForm.getLastUpdate());
			v3xFileGovDocSendForm.setAccountId(edocForm.getDomainId());
			v3xFileDAO.save(v3xFileGovDocSendForm);
			fileId = v3xFileGovDocSendForm.getId();
			File fileOutGovDocSendForm = new File(partitionManager.getFolder(v3xFileGovDocSendForm.getCreateDate(), true)+File.separator+fileId);
			LOG.info("文单路径：《" + edocForm.getName() + "》(" + edocForm.getId() + ")" + fileOutGovDocSendForm.getAbsolutePath());
			try {
				FileUtil.copyFile(xsnFile, fileOutGovDocSendForm);
			} catch (Exception e1) {
				LOG.error("文单：《" + edocForm.getName() + "》(" + edocForm.getId() + ")升级发生异常！");
				LOG.error(e1.getMessage(),e1);
				continue;
			}
			File file = xsnFile;
			try {
				file = this.fileManager.getFile(v3xFileGovDocSendForm.getId(), edocForm.getCreateTime());
				if (file == null) {
					file = xsnFile;
				}else {
					file = this.fileManager.decryptionFile(file);
				}
			} catch (Exception e) {
				file = xsnFile;
			}
			V3XFile v3 = this.fileManager.getV3XFile(v3xFileGovDocSendForm.getId());
	        Map<String, String> opitionSort = new HashMap<String, String>();
	        FormBean fb = null;
			try {
				InfoPathObject xsf = formApi4Cap3.parseXSN(file);
				// infopath的各种校验
				String check = validateInfopath(xsf, formType, -1);
				LOG.info("formType4="+formType);
				if (!"".equals(check)) {
					LOG.error("文单：《" + edocForm.getName() + "》(" + edocForm.getId() + ")升级发生异常！(infopath的各种校验发生异常)");
					LOG.error(check);
					continue;
				}
				// 这里主要是进行表格控件查核,如果查核有问题,则会抛出BusinessException,后面会接住此Exception,并return
				for (int i = 0; i < xsf.getViewList().size(); i++) {
					InfoPath_xsl xsl = xsf.getViewList().get(i);
					xsl.covertContent(null);
				}
				fb = formApi4Cap3.getFormBeanForUpgrade(xsf, formType,edocForm.getDomainId(),opitionSort);
				LOG.info("formType5="+formType);
				// OA-1063 上传infopath表名和实际的不一致
				String fileName = v3.getFilename();

				if (edocForm.getIsSystem()) {
					fb.setFormName(edocForm.getName()+"(老数据预置文单)");
				}else{
					fb.setFormName(edocForm.getName());
				}
				fb.setUseFlag(useFlag);
				fb.setFormType(formType);// 类型
				LOG.info("formType1="+fb.getFormType());
				if(fb.getFormType() == Enums.FormType.govDocSendForm.getKey()|| fb.getFormType() == Enums.FormType.govDocSignForm.getKey()
					||fb.getFormType() == Enums.FormType.govDocReceiveForm.getKey()||fb.getFormType() == Enums.FormType.govDocExchangeForm.getKey()){
					fb.setGovDocFormType(fb.getFormType());
					fb.setFormType(Enums.FormType.processesForm.getKey());
		        }
				// 增加此表单的源xsn文件关系
				List<FormResource> resourceList = (List<FormResource>) fb.getExtraAttr("infoPathResource");
				FormResource fr = new FormResource();
				fr.setId(fileId);
				fr.setFormId(fb.getId());
				fr.setResourceName("name.xsn");
				fr.setPropertyType(FormResourcePropertyTypeEnum.InfoPath.getKey());
				fr.setPropertyName("name.xsn");
				fr.setContent("name.xsn");
				resourceList.add(fr);
				fb.putExtraAttr("infoPathResource", resourceList);
				// formCacheManager.addFormBean(userId, fb);
				fb.setCreateDate(edocForm.getCreateTime());
				fb.setCreatorId(edocForm.getCreateUserId());
			} catch (Exception e) {
				LOG.error("文单：《" + edocForm.getName() + "》(" + edocForm.getId() + ")升级发生异常！");
				LOG.error(e.getMessage(),e);
				continue;
			}
			// 发文单
			// String state = "2"; //state=2
			fb.setCategoryId(categoryId);
			fb.setOwnerId(userId);
			GovDocUtil.saveGovdocExtendAndSort(fb, govdocFormExtendManager, govdocFormOpinionSortManager);
			fb.setState(2);
			fb.putExtraAttr("groupNewAccountId", edocForm.getDomainId() + "");
			List<FormViewBean> viewList = fb.getFormViewList();
			
			try {
				formApi4Cap3.saveOrUpdate(this.getFormPermissionConfig(item, newItem, edocForm.getDomainId(), fb,edocForm));
			} catch (Exception e1) {
				LOG.error("文单：《" + edocForm.getName() + "》(" + edocForm.getId() + ")升级发生异常！");
				LOG.error("文单：《" + edocForm.getName() + "》(" + edocForm.getId() + ")权限设置异常！");
				LOG.error(e1.getMessage(),e1);
				continue;
			}
			//屏蔽--许才兵
//			deeDesignManager.setDeeTask4FormField(fb);
			
			//保存意见设置。
			String optionFormatSet = "";
			Set<EdocFormExtendInfo> infos = edocForm.getEdocFormExtendInfo();
	        if(infos != null) {
	            for(EdocFormExtendInfo info : infos) {
	                if(info.getAccountId().longValue() == edocForm.getDomainId()) {
	                	edocForm.setStatus(info.getStatus());
	                	edocForm.setIsDefault(info.getIsDefault());
	                    //bean.setWebOpinionSet(info.getOptionFormatSet());//公文单配置，JSON字符串
	                    //公文配置详细，和新建保持一致
	                    optionFormatSet = info.getOptionFormatSet();
	                }
	            }
	        }
	        GovdocFormExtend govdocFormExtend = new GovdocFormExtend();
        	govdocFormExtend.setAccountId(edocForm.getDomainId());
        	govdocFormExtend.setId(UUIDLong.longUUID());
        	govdocFormExtend.setFormId(fb.getId());
        	if (!"".equals(optionFormatSet)) {
        		govdocFormExtend.setOptionFormatSet(optionFormatSet);
			}else{
				govdocFormExtend.setOptionFormatSet(FormOpinionConfig.getDefualtConfig());
			}
        	govdocFormExtendManager.saveOrUpdate(govdocFormExtend);
        	
        	List<FormFieldBean> formFieldBeans = fb.getAllFieldBeans();
            List<FormFieldBean> list  = new ArrayList<FormFieldBean>();
            for (FormFieldBean formFieldBean : formFieldBeans) {
            	//如果是意见类型，加入到list中
            	if("edocflowdealoption".equals(formFieldBean.getInputType())){
    						list.add(formFieldBean);
            	}
    		}
            List<String> listStr = new ArrayList<String>();
            for (FormFieldBean formFieldBean : list) {
				listStr.add(formFieldBean.getMappingField());
			}
        	
			try {
				List<FormBoundPerm> processList = EdocHelper.getProcessOpinionByEdocFormId4Upgrade(listStr, edocForm.getId(), edocForm.getType(), edocForm.getDomainId(), false);
				List<GovdocFormOpinionSort> govdocFormOpinionSorts = new ArrayList<GovdocFormOpinionSort>();
				for (FormFieldBean formFieldBean : formFieldBeans) {
					if(formFieldBean.getInputType().equals(FormFieldComEnum.EDOCFLOWDEALOPITION.getKey())){
						GovdocFormOpinionSort govdocFormOpinionSort = null;
						if(Strings.isNotEmpty(processList)){
							for (FormBoundPerm formBoundPerm : processList) {
								if (formBoundPerm.getProcessName() != null && formBoundPerm.getProcessName().equals(formFieldBean.getMappingField())) {
									govdocFormOpinionSort = new GovdocFormOpinionSort();
									govdocFormOpinionSort.setId(UUIDLong.longUUID());
									govdocFormOpinionSort.setIdIfNew();
									govdocFormOpinionSort.setFormId(fb.getId());
									govdocFormOpinionSort.setProcessName(formFieldBean.getName());
									govdocFormOpinionSort.setSortType(formBoundPerm.getSortType());
									govdocFormOpinionSort.setDomainId(edocForm.getDomainId());
									govdocFormOpinionSorts.add(govdocFormOpinionSort);
									
								}
							}
						}
					}
				}
				for (String key : opitionSort.keySet()) {
					GovdocFormOpinionSort govdocFormOpinionSort = new GovdocFormOpinionSort();
					govdocFormOpinionSort.setId(UUIDLong.longUUID());
					govdocFormOpinionSort.setIdIfNew();
					govdocFormOpinionSort.setFormId(fb.getId());
					govdocFormOpinionSort.setProcessName(key);
					govdocFormOpinionSort.setSortType(opitionSort.get(key));
					govdocFormOpinionSort.setDomainId(edocForm.getDomainId());
					govdocFormOpinionSorts.add(govdocFormOpinionSort);
				}
				govdocFormOpinionSortManager.saveOrUpdateList(govdocFormOpinionSorts);
			} catch (Exception e1) {
				LOG.error("文单：《" + edocForm.getName() + "》(" + edocForm.getId() + ")升级发生异常！");
				LOG.error("文单：《" + edocForm.getName() + "》(" + edocForm.getId() + ")文单格式设置异常！");
				LOG.error(e1.getMessage(),e1);
			}
			try {
				if (formApi4Cap3.isNewForm(fb.getId())) {
					if (fb.needCheckTotalNum()) {
						BizValidateResultVO resultVO = formApi4Cap3.validateBiz(BizOperationEnum.create_form_save);
						if (!resultVO.isSuccess()) {
							throw new BizException(resultVO.getCode());
						}
					}
					fb.setSourceStr(formApi4Cap3.getEncodeString(
					        formApi4Cap3.getCreateSourceType(BizOperationEnum.create_form_save), fb.getId()));
				}
				LOG.info("formType2="+fb.getFormType());
				formApi4Cap3.saveOrUpdateFormBean(fb);
				LOG.info("formType3="+fb.getFormType());
				//升级默认文单
				if(isDefaultForm){
					boolean defaultFlag = false;
					if(formType==6 || formType==8 || formType==9){//以前是5 7 8 5被表单动态表占用 Enum.java
						defaultFlag = formApi4Cap3.setDefaultGovdocForm(String.valueOf(fb.getId()), formType, String.valueOf(edocForm.getDomainId()), String.valueOf(categoryId));
					}
					if(!defaultFlag){
						LOG.info("公文单默认升级"+ResourceUtil.getString("form.formlist.setdefaultgovdocfailed"));
			        }
				}
				
			} catch (BizException be) {
				LOG.error("文单：《" + edocForm.getName() + "》(" + edocForm.getId() + ")升级发生异常！");
				LOG.error(be);
				continue;
			} catch (BusinessException e) {
				LOG.error("文单：《" + edocForm.getName() + "》(" + edocForm.getId() + ")升级发生异常！");
				if (e.getCause() != null && e.getCause().getMessage().contains("65535")) {
					// OA-60624 MYSQL字段超出范围 报错控制
					LOG.error(ResourceUtil.getString("form.baseinfo.totalLength.formDesignSave.error"));
					continue;
				} else {
					LOG.error("文单：《" + edocForm.getName() + "》(" + edocForm.getId() + ")升级发生异常！");
					LOG.error(e.getMessage(),e);
					continue;
				}
			}
			edocUpgradeDao.upgradeAIP(edocForm.getId(), fb.getId());
			formIds.put(edocForm.getId(), fb.getId());
			
		}
		LOG.info("公文单升级结束...");
	}

	private FormPermissionConfig getFormPermissionConfig(String oldItem,String newItem,Long unitId,FormBean formBean,EdocForm edocForm) throws Exception{
		if (edocForm == null) {
			FormPermissionConfig formPermissionConfig = new FormPermissionConfig();
			formPermissionConfig.setId(UUIDLong.longUUID());
			formPermissionConfig.setFormId(formBean.getId());
			Map<String, String> map = new HashMap<String, String>();
			FlipInfo flipInfo = new FlipInfo();
	    	flipInfo.setPage(-1);
	    	flipInfo.setSize(-1);
	        List<PermissionVO> p_list = edocUpgradeDao.getPermission(newItem,unitId);
			for (PermissionVO permissionVO : p_list) {
				String relation = null;
				for (FormViewBean formViewBean : formBean.getAllViewList()) {
					if (relation != null) {
						break;
					}
					for (FormAuthViewBean formAuthViewBean : formViewBean.getAllOperations()) {
						if ("niwen".equals(permissionVO.getName()) || "dengji".equals(permissionVO.getName())) {
							if ("add".equals(formAuthViewBean.getType())) {
								relation = formViewBean.getId() + "." + formAuthViewBean.getId() + ".isDefault";
								break;
							}
						}else{
							if ("readonly".equals(formAuthViewBean.getType())) {
								relation = formViewBean.getId() + "." + formAuthViewBean.getId() + ".isDefault";
								break;
							}
						}
					}
				}
				map.put(permissionVO.getFlowPermId().toString(), relation);
			}
			formPermissionConfig.setConfig(JSONUtil.toJSONString(map));
			return formPermissionConfig;
		}
		//老公文节点权限集合
        List<PermissionVO> oldPermissionList = edocUpgradeDao.getPermission(oldItem,unitId);
        //新公文节点权限集合
        List<PermissionVO> newPermissionList = edocUpgradeDao.getPermission(newItem,unitId);
        FormPermissionConfig formPermissionConfig = new FormPermissionConfig();
        formPermissionConfig.setIdIfNew();
        formPermissionConfig.setFormId(formBean.getId());
        Map<String, String> config = new HashMap<String, String>();
        List<FormViewBean> viewList = formBean.getAllViewList();
        if (CollectionUtils.isNotEmpty(viewList)) {
        	FormViewBean view = viewList.get(0);
        	List<FormAuthViewBean> operationList = view.getAllOperations();
        	FormAuthViewBean tempFormAuthViewBean = null;
        	for (FormAuthViewBean operation : operationList) {
        		List<FormAuthViewFieldBean> authViewFieldBeans = operation.getFormAuthorizationFieldList();
        		for (FormAuthViewFieldBean formAuthViewFieldBean : authViewFieldBeans) {
        			if ("add".equals(operation.getType())) {
        				formAuthViewFieldBean.setAccess(FieldAccessType.edit.getKey());
        			} else {
        				formAuthViewFieldBean.setAccess(FieldAccessType.browse.getKey());
        			}
        		}
        		if ("update".equals(operation.getType())) {
        			tempFormAuthViewBean = (FormAuthViewBean) operation.clone();
        		}
        	}
        	for (PermissionVO permission : oldPermissionList) {
        		List<EdocFormFlowPermBound> list = null;
        		if (edocForm != null) {
        			list = edocFormManager.findBoundByFormId(edocForm.getId(), unitId, permission.getName());
        		}
        		List<EdocElementFlowPermAcl> acl_list = edocElementFlowPermAclManager.getEdocElementFlowPermAcls(permission.getFlowPermId());
        		FormAuthViewBean formAuthViewBean = null;
        		if ("niwen".equals(permission.getName())) {
        			for (FormAuthViewBean viewBean : operationList) {
        				if ("add".equals(viewBean.getType())) {
        					formAuthViewBean = viewBean;
        					break;
        				}
        			}
        		}else{
        			formAuthViewBean = (FormAuthViewBean) tempFormAuthViewBean.clone();
        			formAuthViewBean.setNewId();
        			if("fengfa".equals(permission.getName())){
        				formAuthViewBean.setName("分送");
        			}else{
        				formAuthViewBean.setName(permission.getLabel());
        			}
        		}
        		List<FormAuthViewFieldBean> authViewFieldBeans = formAuthViewBean
        				.getFormAuthorizationFieldList();
        		for (FormAuthViewFieldBean formAuthViewFieldBean : authViewFieldBeans) {
        			if (list != null && list.size() > 0 && formAuthViewFieldBean.getFormFieldBean().getInputType().equals(FormFieldComEnum.EDOCFLOWDEALOPITION.getKey())) {
        				for (EdocFormFlowPermBound bound : list) {
        					if (bound.getProcessName().equals(formAuthViewFieldBean.getFormFieldBean().getMappingField())) {
        						formAuthViewFieldBean.setAccess(FieldAccessType.edit.getKey());
        					}
        				}
        			}else if (acl_list != null && acl_list.size() > 0) {
        				for (EdocElementFlowPermAcl acl : acl_list) {
        					if (acl.getEdocElement().getFieldName()
        							.equals(formAuthViewFieldBean.getFormFieldBean().getMappingField())) {
        						formAuthViewFieldBean.setAccess(acl.getAccess() == 1
        								? FieldAccessType.edit.getKey() : FieldAccessType.browse.getKey());
        						List<EdocFormElement> edocFormElements = edocFormElementDao.getEdocFormElementByElementIdAndFormId(Long.valueOf(acl.getEdocElement().getElementId()), edocForm.getId());
        						for (EdocFormElement edocFormElement : edocFormElements) {
        							if (edocFormElement.getRequired()!=null && edocFormElement.getRequired().booleanValue()) {
        								formAuthViewFieldBean.setIsNotNull(1);
        							}
        						}
        						break;
        					} else {
        						formAuthViewFieldBean.setAccess(FieldAccessType.browse.getKey());
        					}
        				}
        			}
        		}
        		for (PermissionVO newPermission : newPermissionList) {
        			if ("niwen".equals(newPermission.getName()) || "dengji".equals(newPermission.getName())) {
        				for (FormAuthViewBean viewBean : operationList) {
        					if ("add".equals(viewBean.getType())) {
        						config.put(newPermission.getFlowPermId().toString(), view.getId() + "." + viewBean.getId() + ".isDefault");
        						break;
        					}
        				}
        				continue;
        			}
        			if (permission.getName().equals(newPermission.getName())) {
        				if (config.get(newPermission.getFlowPermId()) != null) {
							if (permission.getCategory().equals(EnumNameEnum.edoc_qianbao_permission_policy.name())) {
								break;
							}
						}
        				config.put(newPermission.getFlowPermId().toString(), view.getId() + "." + formAuthViewBean.getId() + ".isDefault");
        				break;
        			}
        		}
        		if (!"niwen".equals(permission.getName())) {
        			operationList.add(formAuthViewBean);
        		}
        	}
        	view.setOperations(operationList);
		}
		formPermissionConfig.setConfig(JSONUtil.toJSONString(config));
		return formPermissionConfig;
	}
	
	/**
	 * 检查infopath文件的控件是否有超过950个
	 * 
	 * @param xsf
	 * @return
	 */
	private String checkFieldCount(InfoPathObject xsf) {
		int count = xsf.getIntoxsd().getMasternamelst().size();
		for (int i = 0; i < xsf.getIntoxsd().getTablst().size(); i++) {
			Map ht = (HashMap) xsf.getIntoxsd().getSlavelst().get(i);
			count += ht.size();
		}
		return count <= 950 ? "" : ResourceUtil.getString("form.fielddesign.infopathfiledlengthbigger");
	}
	
	/**
	 * 对上传的infopath中的控件进行查核
	 * 
	 * @param xsf
	 * @param formType
	 *            表单类型
	 * @param formId
	 *            表单id新建的时候为-1,导入infopath修改的时候为表单id
	 * @return
	 */
	private String validateInfopath(InfoPathObject xsf, int formType, long formId) {
		String checks = ResourceUtil.getString("form.fielddesign.fieldcheck");
		StringBuilder sb = new StringBuilder(",");
		String fieldName = "";
		// 校验控件个数
		String checkFileds = checkFieldCount(xsf);
		if (!"".equals(checkFileds)) {
			return checkFileds;
		}

		// 有高级表单插件的情况下,信息管理以及流程表单允许多视图
		if (xsf.getViewList().size() > 1) {
			if (AppContext.hasPlugin("formAdvanced")) {
				if (formType == FormType.baseInfo.getKey()) {
					return ResourceUtil.getString("form.fielddesign.baseolnyallowsigleview");
				} else if (formType == FormType.planForm.getKey()) {
					return ResourceUtil.getString("form.formcreate.basedata.plan.nomultiview.label");
				}
			} else {
				return ResourceUtil.getString("form.formcreate.isnotadvancedform.multiform.abel");
			}
		}
		// 必须有主表字段
		if (xsf.getIntoxsd().getMasternamelst().size() == 0) {
			return ResourceUtil.getString("form.fielddesign.infopathnomaintable");
		}
		// 校验视图名称是否修改
		if (formId != -1L) {
			if (checkDelFormView(xsf, formId)) {
				// 您导入的infopath文件中删除了视图，此操作不允许
				return ResourceUtil.getString("form.fielddesign.viewname.delete.label");
			}
		}
		// 主表字段
		for (int i = 0; i < xsf.getIntoxsd().getMasternamelst().size(); i++) {
			fieldName = ((String) xsf.getIntoxsd().getMasternamelst().get(i)).toLowerCase();
			if (checks.contains("," + fieldName + ",")) {
				return ResourceUtil.getString("form.system.check");
			}
			if (sb.toString().contains("," + fieldName + ",")) {
				return ResourceUtil.getString("form.infopath.sameFiledName");
			}
			sb.append(fieldName).append(",");
		}
		// 从表
		for (int i = 0; i < xsf.getIntoxsd().getTablst().size(); i++) {
			Map ht = (HashMap) xsf.getIntoxsd().getSlavelst().get(i);
			Object[] ob = ht.values().toArray();
			// 从表字段
			for (int j = 0; j < ob.length; j++) {
				fieldName = ((String) ob[j]).toLowerCase();
				if (checks.contains("," + fieldName + ",")) {
					return ResourceUtil.getString("form.system.check");
				}
				if (sb.toString().contains("," + fieldName + ",")) {
					return ResourceUtil.getString("form.infopath.sameFiledName");
				}
				sb.append(fieldName).append(",");
			}
		}
		return "";
	}
	
	/**
	 * 校验表单infopath文件是否删除了视图
	 * 
	 * @param xsf
	 * @param formId
	 * @return
	 */
	private boolean checkDelFormView(InfoPathObject xsf, long formId) {
		boolean isDelete = false;
		FormBean fb = formApi4Cap3.getForm(formId);
		if (fb != null) {
			List<FormViewBean> fvbList = fb.getFormViewList();
			if (fvbList != null) {
				for (FormViewBean formViewBean : fvbList) {
					String name = xsf.getViewFileCaption(formViewBean.getFormViewFileName());
					if (Strings.isBlank(name)) {
						isDelete = true;
						break;
					}
				}
			}
		}
		return isDelete;
	}
	
	public ConfigManager getConfigManager() {
		return configManager;
	}
	public void setConfigManager(ConfigManager configManager) {
		this.configManager = configManager;
	}
	@Override
	public int getUpgradState() {
		//JDBCAgent agent = new JDBCAgent(true);
		int v = UPGRADE_STATE_NEEDTODO_NO;
		//if(NO_NEEDTO_CHECKED){
		if(false){
			return UPGRADE_STATE_NEEDTODO_NO;
		}
        try {
        	ConfigItem configItem = configManager.getConfigItem(9999999888L);
        	if(configItem != null &&  Strings.isNotBlank(configItem.getConfigValue())){
        		v = Integer.parseInt(configItem.getConfigValue());
        	}
        } catch (Exception e) {
        	LOG.error("获取升级状态出错", e);
        } 
        if(v == UPGRADE_STATE_DONE || v == UPGRADE_STATE_NEEDTODO_NO){
        	NO_NEEDTO_CHECKED = true;
        }
		return v;
	}

	@Override
	public void setUpgradeState(int state) {
		try {
			ConfigItem c = configManager.getConfigItem(9999999888L);
			c.setConfigValue(String.valueOf(state));
			configManager.updateConfigItem(c);
		} catch (Exception e) {
			//e.printStackTrace();
			LOG.error(" 修改升级状态出错  "+e.getMessage());
		} 
	}

	@Override
	public String getTemplateStr() throws BusinessException {
		StringBuilder sBuilder = new StringBuilder("模板:");
		List<String> tempList = edocUpgradeDao.getTempList();
		for (String cur:tempList) {
			sBuilder.append(cur).append(",");
		}
		sBuilder.append(",需要手动完成自动分支条件设置！");
		LOG.info(sBuilder.toString());
		return sBuilder.toString();
	}

	public void setEdocUpgradeDao(UpgradeDao edocUpgradeDao) {
		this.edocUpgradeDao = edocUpgradeDao;
	}

	public void setOrgManager(OrgManager orgManager) {
		this.orgManager = orgManager;
	}

	public void setV3xFileDAO(V3XFileDAO v3xFileDAO) {
		this.v3xFileDAO = v3xFileDAO;
	}

	public void setFileManager(FileManager fileManager) {
		this.fileManager = fileManager;
	}

	public void setFormApi4Cap3(FormApi4Cap3 formApi4Cap3) {
		this.formApi4Cap3 = formApi4Cap3;
	}

	public void setGovdocFormExtendManager(GovdocFormExtendManager govdocFormExtendManager) {
		this.govdocFormExtendManager = govdocFormExtendManager;
	}

	public void setGovdocFormOpinionSortManager(GovdocFormOpinionSortManager govdocFormOpinionSortManager) {
		this.govdocFormOpinionSortManager = govdocFormOpinionSortManager;
	}

	public void setEdocFormElementDao(EdocFormElementDao edocFormElementDao) {
		this.edocFormElementDao = edocFormElementDao;
	}

	public void setEdocFormManager(EdocFormManager edocFormManager) {
		this.edocFormManager = edocFormManager;
	}

	public void setEdocElementFlowPermAclManager(EdocElementFlowPermAclManager edocElementFlowPermAclManager) {
		this.edocElementFlowPermAclManager = edocElementFlowPermAclManager;
	}

	public void setPrivilegeMenuManager(PrivilegeMenuManager privilegeMenuManager) {
		this.privilegeMenuManager = privilegeMenuManager;
	}
}
