/**
 * Author : xuqw
 *   Date : 2015年9月21日 下午6:08:23
 *
 * Copyright (C) 2015 Seeyon, Inc. All rights reserved.
 *
 * This software is the proprietary information of Seeyon, Inc.
 * Use is subject to license terms.
 */
package com.seeyon.apps.edoc.test.api;

import org.apache.log4j.Logger;

import com.seeyon.apps.edoc.test.utils.SeeyonTestUtil;
import com.seeyon.ctp.common.AppContext;
import com.seeyon.ctp.common.GlobalNames;
import com.seeyon.ctp.common.authenticate.domain.User;
import com.seeyon.ctp.common.exceptions.BusinessException;
import com.seeyon.ctp.organization.bo.V3xOrgMember;
import com.seeyon.ctp.organization.manager.OrgManager;
import com.seeyon.ctp.organization.manager.RoleManager;
import com.seeyon.v3x.edoc.manager.EdocHelper;


/**
 * 会议单元测试初始化
 * @author xuqw
 *
 */
public class EdocApiImplDataUtil {

    private static final Logger logger = Logger.getLogger(EdocApiImplDataUtil.class);
    
    private static OrgManager orgManager = null;
    private static RoleManager roleManager = null;
    
    static{
        orgManager = (OrgManager)AppContext.getBean("orgManager");
        roleManager = (RoleManager)AppContext.getBean("roleManager");
    }
    
    
    /**
     * 初始化数据
     * 
     * @param initdb
     *            是否初始化数据库
     * @throws BusinessException
     */
    public static void initData(boolean initdb) throws BusinessException {

        
        if (initdb) {
                
            //授权
            V3xOrgMember edoc1 = orgManager.getMemberByLoginName("edoc1");
            
          //管理员
            V3xOrgMember admin = orgManager.getMemberByLoginName("seeyon");
            User user = SeeyonTestUtil.createSessionUser(admin);
            AppContext.putThreadContext(GlobalNames.SESSION_CONTEXT_USERINFO_KEY, user);
            
            //登记权限
            String exchangeRole = null;
            if(!EdocHelper.isG6Version()){
                exchangeRole = "RecEdoc";
            }else{
                exchangeRole = "RegisterEdoc";
            }
            boolean flag = orgManager.isRole(edoc1.getId(), edoc1.getOrgAccountId(), exchangeRole);
            if(!flag){
                roleManager.batchRole2Member(exchangeRole, AppContext.currentAccountId(), edoc1.getId().toString());
            }
            
            //发文拟文
            boolean SendEdoc_role = orgManager.isRole(edoc1.getId(), edoc1.getOrgAccountId(), "SendEdoc");
            if(!SendEdoc_role){
                roleManager.batchRole2Member("SendEdoc", AppContext.currentAccountId(), edoc1.getId().toString());
            }
            
            
            if(EdocHelper.isG6Version()){
                
              //G6分发，收文拟文
                boolean FenfaEdoc_role = orgManager.isRole(edoc1.getId(), edoc1.getOrgAccountId(), "FenfaEdoc");
                if(!FenfaEdoc_role){
                    roleManager.batchRole2Member("FenfaEdoc", AppContext.currentAccountId(), edoc1.getId().toString());
                }
            }
            
          //签报拟文
            boolean SignEdoc_role = orgManager.isRole(edoc1.getId(), edoc1.getOrgAccountId(), "SignEdoc");
            if(!SignEdoc_role){
                roleManager.batchRole2Member("SignEdoc", AppContext.currentAccountId(), edoc1.getId().toString());
            }
            
            //单位收发员
            boolean Accountexchange_role = orgManager.isRole(edoc1.getId(), edoc1.getOrgAccountId(), "Accountexchange");
            if(!Accountexchange_role){
                roleManager.batchRole2Member("Accountexchange", AppContext.currentAccountId(), edoc1.getId().toString());
            }
            
            //部门收发员
            V3xOrgMember edoc2 = orgManager.getMemberByLoginName("edoc2");
            boolean Departmentexchange_role = orgManager.isRole(edoc2.getId(), edoc2.getOrgAccountId(), "Departmentexchange");
            if(!Departmentexchange_role){
                roleManager.batchRole2Member("Departmentexchange", AppContext.currentAccountId(), edoc2.getId().toString());
            }
            
            
            AppContext.removeThreadContext(GlobalNames.SESSION_CONTEXT_USERINFO_KEY);
        }
    }
}


