/**
 * Author : xuqw
 *   Date : 2015年9月27日 下午11:16:13
 *
 * Copyright (C) 2015 Seeyon, Inc. All rights reserved.
 *
 * This software is the proprietary information of Seeyon, Inc.
 * Use is subject to license terms.
 */
package com.seeyon.apps.edoc.test.api;

import java.io.File;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import javax.servlet.http.HttpSession;

import org.apache.log4j.Logger;
import org.springframework.web.servlet.ModelAndView;

import com.seeyon.apps.edoc.test.constants.EdocContant;
import com.seeyon.apps.edoc.test.http.SeeyonTestRequest;
import com.seeyon.apps.edoc.test.http.SeeyonTestResponse;
import com.seeyon.apps.edoc.test.utils.ControllerFactory;
import com.seeyon.apps.edoc.test.utils.SeeyonTestUtil;
import com.seeyon.ctp.common.AppContext;
import com.seeyon.ctp.common.GlobalNames;
import com.seeyon.ctp.common.authenticate.domain.User;
import com.seeyon.ctp.common.config.manager.ConfigManager;
import com.seeyon.ctp.common.exceptions.BusinessException;
import com.seeyon.ctp.common.permission.manager.PermissionManager;
import com.seeyon.ctp.common.po.config.ConfigItem;
import com.seeyon.ctp.common.po.template.CtpTemplate;
import com.seeyon.ctp.common.template.manager.TemplateManager;
import com.seeyon.ctp.organization.OrgConstants;
import com.seeyon.ctp.organization.bo.MemberRole;
import com.seeyon.ctp.organization.bo.V3xOrgMember;
import com.seeyon.ctp.organization.bo.V3xOrgRole;
import com.seeyon.ctp.organization.manager.OrgManager;
import com.seeyon.ctp.privilege.bo.PrivMenuBO;
import com.seeyon.ctp.privilege.manager.MenuManager;
import com.seeyon.ctp.util.Strings;
import com.seeyon.ctp.util.UUIDLong;
import com.seeyon.ctp.util.XMLCoder;
import com.seeyon.ctp.util.json.JSONUtil;
import com.seeyon.ctp.workflow.exception.BPMException;
import com.seeyon.ctp.workflow.vo.CPMatchResultVO;
import com.seeyon.ctp.workflow.wapi.WorkflowApiManager;
import com.seeyon.v3x.edoc.controller.EdocController;
import com.seeyon.v3x.edoc.controller.EdocListController;
import com.seeyon.v3x.edoc.domain.EdocForm;
import com.seeyon.v3x.edoc.domain.EdocSummary;
import com.seeyon.v3x.edoc.manager.EdocFormManager;
import com.seeyon.v3x.edoc.webmodel.EdocSummaryModel;

import net.joinwork.bpm.engine.wapi.WorkflowBpmContext;


/**
 * <p>Title       : 应用模块名称</p>
 * <p>Description : 代码描述</p>
 * <p>Copyright   : Copyright (c) 2015</p>
 * <p>Company     : seeyon.com</p>
 */
public class EdocSummaryDataUtil {
    
    private static final Logger logger = Logger.getLogger(EdocSummaryDataUtil.class);
    
    //create meeting fields...
    

    ///默认的会议值
    
    
    //Spring 相关的Bean
    private static OrgManager orgManager = null;
    private static EdocController edocController = null;
    private static EdocListController edocListController = null;
    private static ControllerFactory factory = ControllerFactory.getInstance();
    private static PermissionManager permissionManager = null;
    private static ConfigManager configManager = null;
    private static EdocFormManager edocFormManager = null;;
    private static MenuManager menuManager = null;
    private static TemplateManager templateManager = null;
    private static WorkflowApiManager wapi    = null;
    
    
    //全局的常量
    /*在ctp-test初始化的时候加载的*/
    public final static String TEST_HOME = System.getProperty("TEST_HOME");
    
    public final static String testResponseFolder = TEST_HOME + "/home/tomcat/webapp/logs/";
    
    /*模拟response的输出流，让流直接操作文件*/
    public final static String testResponseFile = testResponseFolder + "edoc.txt";
    
    private static Map<String, String> defaultFieldMap = null;
    
    static{
        
        orgManager = (OrgManager) AppContext.getBean("orgManager");
        edocController = factory.generateController(EdocController.class);
        edocListController = factory.generateController(EdocListController.class);
        permissionManager = (PermissionManager)AppContext.getBean("permissionManager");
        configManager = (ConfigManager)AppContext.getBean("configManager");
        edocFormManager = (EdocFormManager)AppContext.getBean("edocFormManager");
        menuManager = (MenuManager)AppContext.getBean("menuManager");
        templateManager = (TemplateManager) AppContext.getBean("templateManager");
        wapi    = (WorkflowApiManager) AppContext.getBean("wapi");
        
        File responseFolder = new File(testResponseFolder);
        if(!responseFolder.exists()){
            responseFolder.mkdirs();
        }
        
        try {
            createDefualtMap();
        } catch (BusinessException e) {
            logger.error("创建会议默认的Map初始化失败", e);
        }
    }
    
    /**
     * 加载人员资源菜单
     * @Author      : xuqw
     * @Date        : 2016年5月27日下午3:01:57
     * @param memberId
     * @param accountId
     */
    public static void addMemberMenu(long memberId, long accountId){
        try {
        //人员菜单
          Map<Long, PrivMenuBO> result = null;
          List<MemberRole> roles = orgManager.getMemberRoles(memberId, null);
          if (null != roles) {
              Set<Long> roleIds = new HashSet<Long>();
  
              for (MemberRole memberRole : roles) {
                  V3xOrgRole role = memberRole.getRole();
                  if (role != null) {
                      if (role.getBond() == OrgConstants.ROLE_BOND.BUSINESS.ordinal()) {
                          //业务生成器的留下
                      } else if (Strings.equals(role.getOrgAccountId(), OrgConstants.GROUPID)) {
                          //集团的留下
                      } else if (Strings.equals(role.getOrgAccountId(), accountId)) {
                          //当前单位的留下
                      } else {
                          continue;
                      }
  
                      roleIds.add(role.getId());
                  }
              }
  
              result = menuManager.getByRole(roleIds.toArray(new Long[roleIds.size()]));
          }
          String userResourceKey = "Menu-"+memberId+"-" + accountId;
          AppContext.putThreadContext(userResourceKey, result);
          HttpSession session = (HttpSession)AppContext.getThreadContext(GlobalNames.THREAD_CONTEXT_SESSION_KEY);
          AppContext.putSessionContext(userResourceKey, result);
          
          //集团版报错
          String groupResourceKey = "Menu-"+memberId+"-" + OrgConstants.GROUPID;
          AppContext.putThreadContext(groupResourceKey, new HashMap<Long, PrivMenuBO>());
          AppContext.putSessionContext(groupResourceKey, new HashMap<Long, PrivMenuBO>());
          
        } catch (BusinessException e) {
            logger.error("设置人员熟悉失败", e);
        }
    }
    
    /**
     * 移除资源
     * @Author      : xuqw
     * @Date        : 2016年5月27日下午12:22:17
     * @param user
     */
    public static void clearMemberMenu(long memberId, long accountId) {
        
        String userResourceKey = "Menu-"+memberId+"-" + accountId;
        AppContext.removeThreadContext(userResourceKey);
        AppContext.removeSessionArrribute(userResourceKey);
        
        
        String groupResourceKey = "Menu-"+memberId+"-" + OrgConstants.GROUPID;
        AppContext.removeThreadContext(groupResourceKey);
        AppContext.removeSessionArrribute(groupResourceKey);
    }
    
    /**
     * 节点权限添加资源
     * @Author      : xuqw
     * @Date        : 2016年5月27日下午12:21:57
     * @param user
     */
    private static void addUserResource(User user){
        
        try {
            
            Map<Long, PrivMenuBO> resources = menuManager.getByMember(user.getId(), user.getLoginAccount());
            List<String> resList = new ArrayList<String>();
            for (PrivMenuBO res : resources.values()) {
                String code = res.getResourceCode();
                if(code == null){
                    continue;
                }
                resList.add(code);
            }
            user.setProperty("resourceJsonStr", JSONUtil.toJSONString(resList));
            
        } catch (BusinessException e) {
            logger.error("设置人员熟悉失败", e);
        }
    }
    
    /**
     * 创建带附件的快速发文公文
     * @Author      : xuqw
     * @Date        : 2015年10月16日上午12:31:47
     * @param subject
     * @param attrs
     * @return
     * @throws Exception
     */
    public static EdocSummary createQuickEdocSummaryWithAttatchmentParam(String subject, String attrs) throws Exception{
        
        V3xOrgMember edoc1 = orgManager.getMemberByLoginName("edoc1");
        
        Date nowDate = new Date();
        Long id = UUIDLong.longUUID();
        DateFormat df = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        
        User user = SeeyonTestUtil.createSessionUser(edoc1);
        addMemberMenu(user.getId(), user.getAccountId());
        addUserResource(user);
        AppContext.putThreadContext(GlobalNames.SESSION_CONTEXT_USERINFO_KEY, user);
        
        EdocForm edocForm = edocFormManager.getDefaultEdocForm(edoc1.getOrgAccountId(), 0);
        
        //Permission permission = permissionManager.getPermission("edoc_send_permission_policy", "niwen", edoc1.getOrgAccountId());
        ConfigItem item = configManager.getConfigItem("edoc_send_permission_policy","niwen", edoc1.getOrgAccountId());
        String rParams = "workflow_last_input=false"
                + "&sendEdocId="
                + "&taohongriqiSwitch=true"
                + "&forwordtosend_recAffairId="
                + "&forwardMember="
                + "&waitRegister_recieveId="
                + "&isAgent="
                + "&agentToId="
                + "&agentToName="
                + "&process_desc_by="
                + "&process_xml="
                + "&process_rulecontent="
                + "&readyObjectJSON="
                + "&process_subsetting="
                + "&moduleType="
                + "&workflow_newflow_input="
                + "&workflow_node_peoples_input="
                + "&workflow_node_condition_input="
                + "&processId=&caseId=-1"
                + "&subObjectId="
                + "&currentNodeId="
                + "&templeteProcessId="
                + "&edocType_mark=0"
                + "&backBoxToEdit="
                + "&recieveId="
                + "&registerId=-1"
                + "&distributeEdocId=" + id
                + "&forwordType="
                + "&app="
                + "&openFrom="
                + "&edocGovType="
                + "&subType="
                + "&relationRecd=isNot"
                + "&relationRecId="
                + "&relationSend=isNot"
                + "&recSummaryIdVal=-1"
                + "&pageview="
                + "&currContentNum=0"
                + "&contentNo=0"
                + "&isUniteSend=false"
                + "&appName=4"
                + "&comm=new_form"
                + "&id=" + id
                + "&forwordtosend="
                + "&summary_id=" + id
                + "&summaryId=" + id
                + "&newSummaryId="
                + "&edocType=0"
                + "&exchangeId="
                + "&actorId=" + item.getId()
                + "&returnDeptId="
                + "&currContentNum=0"
                + "&templeteId="
                + "&currentNodeId=start"
                + "&supervisorId="
                + "&supervisors="
                + "&unCancelledVisor="
                + "&sVisorsFromTemplate="
                + "&awakeDate="
                + "&superviseTitle="
                + "&fromSend="
                + "&loginAccountId=" + edoc1.getOrgAccountId()
                + "&orgAccountId=" + edoc1.getOrgAccountId()
                + "&strEdocId="
                + "&__ActionToken=SEEYON_A8"
                + "&archiveId="
                + "&hasArchive=false"
                + "&popJsonId="
                + "&popNodeSelected="
                + "&popNodeCondition="
                + "&popNodeNewFlow="
                + "&allNodes="
                + "&nodeCount="
                + "&isModifyContent=0"
                + "&isModifyForm=0"
                + "&isModifyAtt=0"
                + "&draftChoose="
                + "&affairId="
                + "&checkOption="
                + "&docMarkValue="
                + "&docMarkValue2="
                + "&docInmarkValue="
                
                //文单字段
                + "&my:send_unit_id=Account|" + edoc1.getOrgAccountId()
                + "&my:send_to_id="
                + "&my:copy_to_id="
                + "&my:report_to_id="
                + "&my:send_unit_id2="
                + "&my:send_to_id2="
                + "&my:copy_to_id2="
                + "&my:report_to_id2="
                + "&my:print_unit_id="
                + "&my:send_department_id=Department|" + edoc1.getOrgDepartmentId()
                + "&my:send_department_id2="
                + "&my:undertakenoffice_id="
                + "&my:doc_mark=创世纪"+id+"号"
                + "&my:doc_mark=创世纪"+id+"号"
                + "&my:urgent_level=1"
                + "&my:secret_level=0"
                + "&my:send_unit=" + EdocContant.TEST_ACCOUNT_NAME
                + "&my:signing_date="
                + "&my:create_person=" + edoc1.getName()
                + "&my:phone="
                + "&my:copies="
                + "&my:subject=" + subject
                + "&my:send_to="
                + "&my:copy_to="
                + "&my:attachments="
                
                + "&content="
                + "&bodyType=HTML"
                + "&bodyCreateDate=" + df.format(nowDate)
                + "&contentName="
                + "&_a8_no_cache="
                + "&isQuickSend=on"
                + "&workflowInfo=<点击新建流程>"
                + "&exchangeDeptType=Creater"
                + "&edocExchangeType=1"
                + "&memberList=" + edoc1.getId()
                + "&deadline2=-1"
                + "&deadlineSelect=0"
                + "&deadLineDateTimeInput="
                + "&deadLineDateTime="
                + "&deadline4temp="
                + "&deadlineTime="
                + "&advanceRemind2=0"
                + "&advanceRemind=0"
                + "&edocCategorySelect=all"
                + "&edoctable=" + edocForm.getId()
                + "&isTrack=1"
                + "&trackRange=1"
                + "&trackMembers="
                + "&zdgzryName="
                + "&kuaisuGuidangSelect=1"
                + "&submitstr="
                + "&policy=niwen"
                + "&note="
                + attrs;

        // 初始化Request和Response
        SeeyonTestRequest request = new SeeyonTestRequest();
        request.setParams(rParams);

        SeeyonTestResponse response = new SeeyonTestResponse(testResponseFile);

        // 设置session信息
         AppContext.putThreadContext(GlobalNames.THREAD_CONTEXT_REQUEST_KEY, request);

        // 调用Contraller
         edocController.send(request, response);

        // 数据销毁
        AppContext.removeThreadContext(GlobalNames.THREAD_CONTEXT_REQUEST_KEY);
        SeeyonTestUtil.closeResponseIO(response);
    
        
        //获取刚刚保存的EdocSummary
        rParams = "listType=listSent"
                + "&track="
                + "&edocType=0"
                + "&appName=4"
                + "&condition=subject"
                + "&textfield=" + subject;
        SeeyonTestRequest request2 = new SeeyonTestRequest();
        request2.setParams(rParams);

        SeeyonTestResponse response2 = new SeeyonTestResponse(testResponseFile);

        // 设置session信息
         AppContext.putThreadContext(GlobalNames.THREAD_CONTEXT_REQUEST_KEY, request2);

        // 调用Contraller
         ModelAndView listView = edocListController.listSent(request2, response2);
         
         // 数据销毁
         AppContext.removeThreadContext(GlobalNames.THREAD_CONTEXT_REQUEST_KEY);
         AppContext.removeThreadContext(GlobalNames.SESSION_CONTEXT_USERINFO_KEY);
         clearMemberMenu(user.getId(), user.getAccountId());
         SeeyonTestUtil.closeResponseIO(response2);
         
         Map<String, Object> listValus = listView.getModel();
         
         List<EdocSummaryModel> queryList = (List<EdocSummaryModel>) listValus.get("pendingList");
         
         return queryList.get(0).getSummary();
    }
    
    /**
     * 快速发文，没有流程数据
     * @Author      : xuqw
     * @Date        : 2015年10月13日下午11:05:23
     * @param subject
     * @throws Exception
     */
    public static EdocSummary createQuickEdocSummary(String subject) throws Exception{
        return createQuickEdocSummaryWithAttatchmentParam(subject, "");
    }
    
    /**
     * 快速发文，没有流程数据
     * @Author      : xuqw
     * @Date        : 2015年10月13日下午11:05:23
     * @param subject
     * @throws Exception
     */
    public static EdocSummary createQuickEdocSummaryWithAttatchment(String subject) throws Exception{
        
        String attrs = "&attachment_id="
                + "&attachment_reference="
                + "&attachment_subReference="
                + "&attachment_category="
                + "&attachment_type=0"
                + "&attachment_filename=" + subject + "附件"
                + "&attachment_mimeType=application/octet-stream"
                + "&attachment_createDate=2015-10-16 00:25:34"
                + "&attachment_size=6860"
                + "&attachment_fileUrl=10000001"
                + "&attachment_description="
                + "&attachment_needClone=false"
                + "&attachment_extReference="
                + "&attachment_extSubReference="
                + "&input_file_id=10000001";
        return createQuickEdocSummaryWithAttatchmentParam(subject, attrs);
    }
    
    
    
    /**
     * 创建公文, 传入特殊的流程数据
     * @Author      : xuqw
     * @Date        : 2015年10月15日下午1:53:56
     * @param subject
     * @param process
     * @return
     * @throws Exception 
     */
    private static EdocSummary createSimpleEdocSummary(String subject, String process) throws Exception{

        V3xOrgMember edoc1 = orgManager.getMemberByLoginName("edoc1");
        
        Date nowDate = new Date();
        Long id = UUIDLong.longUUID();
        DateFormat df = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        
        //设置当前登录帐号
        User user = AppContext.getCurrentUser();
        if(user == null){
            user = SeeyonTestUtil.createSessionUser(edoc1);
            AppContext.putThreadContext(GlobalNames.SESSION_CONTEXT_USERINFO_KEY, user);
        }
        
        //资源权限
        addMemberMenu(user.getId(), user.getAccountId());
        addUserResource(user);
        
        AppContext.putThreadContext(GlobalNames.SESSION_CONTEXT_USERINFO_KEY, user);
        
        EdocForm edocForm = edocFormManager.getDefaultEdocForm(edoc1.getOrgAccountId(), 0);
        
        //Permission permission = permissionManager.getPermission("edoc_send_permission_policy", "niwen", edoc1.getOrgAccountId());
        ConfigItem item = configManager.getConfigItem("edoc_send_permission_policy","niwen", edoc1.getOrgAccountId());
        String rParams = "workflow_last_input=false"
                + "&sendEdocId="
                + "&taohongriqiSwitch=true"
                + "&forwordtosend_recAffairId="
                + "&forwardMember="
                + "&waitRegister_recieveId="
                + "&isAgent="
                + "&agentToId="
                + "&agentToName="
                + "&process_desc_by=xml"
                
                + process
                
                + "&process_rulecontent="
                + "&readyObjectJSON="
                + "&process_subsetting="
                + "&moduleType="
                + "&workflow_newflow_input="
                + "&workflow_node_peoples_input="
                + "&processId=" 
                + "&caseId=-1" 
                + "&subObjectId="
                + "&currentNodeId=" 
                + "&templeteProcessId="
                + "&edocType_mark=0" 
                + "&backBoxToEdit=" 
                + "&recieveId="
                + "&registerId=-1" 
                + "&distributeEdocId=" + id
                + "&forwordType=" 
                + "&app=" 
                + "&openFrom=" 
                + "&edocGovType="
                + "&subType=" 
                + "&relationRecd=isNot" 
                + "&relationRecId="
                + "&relationSend=isNot" 
                + "&recSummaryIdVal=-1" 
                + "&pageview="
                + "&currContentNum=0" 
                + "&contentNo=0" 
                + "&isUniteSend=false"
                + "&appName=4" 
                + "&comm=new_form" 
                + "&id=" + id
                + "&forwordtosend=" 
                + "&summary_id=" + id
                + "&summaryId=" + id 
                + "&newSummaryId="
                + "&edocType=0" 
                + "&exchangeId="
                + "&actorId=" + item.getId()
                + "&returnDeptId="
                + "&currContentNum=0" 
                + "&templeteId=" 
                + "&currentNodeId=start"
                + "&supervisorId=" 
                + "&supervisors=" 
                + "&unCancelledVisor="
                + "&sVisorsFromTemplate=" 
                + "&awakeDate=" 
                + "&superviseTitle="
                + "&fromSend=" 
                + "&loginAccountId=" + edoc1.getOrgAccountId()
                + "&orgAccountId=" +  + edoc1.getOrgAccountId() 
                + "&strEdocId="
                + "&__ActionToken=SEEYON_A8" 
                + "&archiveId="
                + "&hasArchive=false" 
                + "&popJsonId=" 
                + "&popNodeSelected="
                + "&popNodeCondition="
                + "&popNodeNewFlow=" 
                + "&allNodes="
                + "&nodeCount=" 
                + "&isModifyContent=0" 
                + "&isModifyForm=0"
                + "&isModifyAtt=0" 
                + "&draftChoose=" 
                + "&affairId="
                + "&checkOption=" 
                + "&docMarkValue=" 
                + "&docMarkValue2="
                + "&docInmarkValue="
                + "&content=" 
                + "&bodyType=HTML"
                + "&bodyCreateDate=" + df.format(nowDate) 
                + "&contentName="
                + "&_a8_no_cache=" 
                + "&workflowInfo="+edoc1.getName()+"(审批)"
                + "&edocExchangeType=0" 
                + "&exchangeDeptType=Creater"
                + "&memberList=" 
                + "&deadline2=-1"
                + "&deadlineSelect=0"
                + "&deadLineDateTimeInput=" 
                + "&deadLineDateTime="
                + "&deadline4temp=" 
                + "&deadlineTime=" 
                + "&advanceRemind2=0"
                + "&advanceRemind=0"
                + "&edocCategorySelect=" + edocForm.getSubType()
                + "&edoctable=" + edocForm.getId() 
                + "&isTrack=1"
                + "&trackRange=1"
                + "&trackMembers=" 
                + "&zdgzryName="
                + "&kuaisuGuidangSelect=1"
                
                + "&my:send_unit_id=Account|" + edoc1.getOrgAccountId()
                + "&my:send_to_id=" 
                + "&my:copy_to_id="
                + "&my:report_to_id="
                + "&my:send_unit_id2=" 
                + "&my:send_to_id2="
                + "&my:copy_to_id2=" 
                + "&my:report_to_id2="
                + "&my:print_unit_id="
                + "&my:send_department_id=Department|" + edoc1.getOrgDepartmentId()
                + "&my:send_department_id2=" 
                + "&my:undertakenoffice_id="
                + "&my:doc_mark=创世纪"+id+"号"
                + "&my:doc_mark_autocomplete=创世纪"+id+"号"
                + "&my:urgent_level=1" 
                + "&my:secret_level=1"
                + "&my:send_unit=" + EdocContant.TEST_ACCOUNT_NAME 
                + "&my:signing_date="
                + "&my:create_person=" + edoc1.getName() 
                + "&my:phone="
                + "&my:copies="
                + "&my:subject=" + subject
                + "&my:send_to=" 
                + "&my:copy_to="
                + "&my:attachments=" 
                
                + "&submitstr="
                + "&policy=niwen"
                + "&note=";

        // 初始化Request和Response
        SeeyonTestRequest request = new SeeyonTestRequest();
        request.setParams(rParams);

        SeeyonTestResponse response = new SeeyonTestResponse(testResponseFile);

        // 设置session信息
         AppContext.putThreadContext(GlobalNames.THREAD_CONTEXT_REQUEST_KEY, request);

        // 调用Contraller
         edocController.send(request, response);

        // 数据销毁
        AppContext.removeThreadContext(GlobalNames.THREAD_CONTEXT_REQUEST_KEY);
        SeeyonTestUtil.closeResponseIO(response);
        
      //获取刚刚保存的EdocSummary
        rParams = "listType=listSent"
                + "&track="
                + "&edocType=0"
                + "&appName=4"
                + "&condition=subject"
                + "&textfield=" + subject;
        SeeyonTestRequest request2 = new SeeyonTestRequest();
        request2.setParams(rParams);

        SeeyonTestResponse response2 = new SeeyonTestResponse(testResponseFile);

        // 设置session信息
         AppContext.putThreadContext(GlobalNames.THREAD_CONTEXT_REQUEST_KEY, request2);

        // 调用Contraller
         ModelAndView listView = edocListController.listSent(request2, response2);
         
         // 数据销毁
         AppContext.removeThreadContext(GlobalNames.THREAD_CONTEXT_REQUEST_KEY);
         AppContext.removeThreadContext(GlobalNames.SESSION_CONTEXT_USERINFO_KEY);
         
         //移除资源
         clearMemberMenu(user.getId(), user.getAccountId());
         
         SeeyonTestUtil.closeResponseIO(response2);
         
         Map<String, Object> listValus = listView.getModel();
         
         List<EdocSummaryModel> queryList = (List<EdocSummaryModel>) listValus.get("pendingList");
         
         return queryList.get(0).getSummary();
    }
    
    /**
     * 创建一条普通的公文， 发起人是edoc1
     * @return 
     * @Author      : xuqw
     * @Date        : 2015年10月13日下午3:11:21
     * @throws Exception
     */
    public static EdocSummary createSimpleEdocSummary(String subject) throws Exception {

        V3xOrgMember edoc1 = orgManager.getMemberByLoginName("edoc1");
        
        int key = 0;
        long nodeId = SeeyonTestUtil.get14Id(key++);
        
        String process =  "<ps>"
                + "<p s=\"\" u=\"\" y=\"0\" x=\"0\" i=\"\" n=\"\" d=\"\" t=\"\">"
                + "<n m=\"0\" y=\"\" x=\"\" t=\"8\" d=\"\" n=\"\" i=\"start\">"
                + "<a a=\"\" b=\"\" c=\"1\" d=\"\" e=\"\" f=\"\" g=\"\" i=\"false\" j=\"false\" k=\"\"/>"
                + "<s c=\"1\" r=\"\" e=\"\" f=\"\" g=\"0\" j=\"multiple\" k=\"\" l=\"\" m=\"false\" s=\"success\" d=\"\" t=\"\" n=\"\" i=\"shenpi\"/>"
                + "</n>"
                + "<n m=\"0\" y=\"0\" x=\"0\" t=\"4\" d=\"\" n=\"end\" i=\"end\">"
                + "<a a=\"\" b=\"\" c=\"1\" d=\"\" e=\"\" f=\"\" g=\"\" i=\"false\" j=\"false\" k=\"\"/>"
                + "<s c=\"1\" r=\"\" e=\"\" f=\"\" g=\"0\" j=\"multiple\" k=\"\" l=\"\" m=\"false\" s=\"success\" d=\"\" t=\"\" n=\"审批\" i=\"shenpi\"/>"
                + "</n>"
                + "<n a=\"1\" b=\"normal\" c=\"false\" l=\"1000\" e=\"0\" t=\"6\" d=\"\" n=\""+edoc1.getName()+"\" i=\""+nodeId+"\">"
                + "<a a=\"\" b=\""+edoc1.getOrgAccountId()+"\" c=\"1\" d=\""+edoc1.getName()+"\" e=\"\" f=\""+edoc1.getId()+"\" h=\"false\" g=\"user\" i=\"false\" j=\"false\" k=\"roleadmin\"/>"
                + "<s c=\"1\" r=\"\" e=\"\" f=\"\" g=\"0\" j=\"all\" k=\"\" l=\"\" m=\"false\" s=\"success\" d=\"\" t=\"\" n=\"审批\" i=\"shenpi\"/>"
                + "</n>"
                + "<l a=\"\" b=\"\" c=\"\" e=\"\" m=\"\" h=\"3\" o=\"0\" j=\"" + nodeId + "\" k=\"start\" t=\"11\" d=\"\" n=\"\" i=\""+SeeyonTestUtil.get14Id(key++)+"\"/>"
                + "<l a=\"\" b=\"\" c=\"\" e=\"\" m=\"\" h=\"3\" o=\"0\" j=\"end\" k=\"" + nodeId + "\" t=\"11\" d=\"\" n=\"\" i=\""+SeeyonTestUtil.get14Id(key++)+"\"/>"
                + "</p>"
                + "</ps>";
        
        User user = SeeyonTestUtil.createSessionUser(edoc1);
        AppContext.putThreadContext(GlobalNames.SESSION_CONTEXT_USERINFO_KEY, user);
        long casheId = doWfPreSubmit(process, "sendEdoc", edoc1);
        
        process = "&process_xml=" + process + "&workflow_node_condition_input={\"matchRequestToken\":\""+ casheId +"\",\"condition\":[{\"nodeId\":\"" + nodeId + "\",\"isDelete\":\"false\"}]}";
        
        return createSimpleEdocSummary(subject, process);
    }
    
    
    /**
     * 预提交工作流， 返回缓存ID
     * 
     * @return
     * @throws BPMException 
     *
     * @Since A8-V5 6.1
     * @Author      : xuqw
     * @Date        : 2017年2月22日下午5:56:28
     *
     */
    private static long doWfPreSubmit(String process, String appName, V3xOrgMember member) throws BPMException{
        
        Long cacheId = UUIDLong.longUUID();
        
        WorkflowBpmContext context = new WorkflowBpmContext();
        context.setAppName(appName);
        context.setProcessXml(process);
        context.setProcessId("");
        context.setCaseId(-1);
        context.setCurrentActivityId("start");
        context.setCurrentWorkitemId(-1);
        context.setCurrentUserId(member.getId().toString());
        context.setCurrentAccountId(member.getOrgAccountId().toString());
        context.setFormData("{\"doc_mark\":{\"value\":\"\",\"type\":\"int\"},\"secret_level\":{\"value\":\"\",\"type\":\"int\"},\"urgent_level\":{\"value\":\"\",\"type\":\"int\"},\"signing_date\":{\"value\":\"\",\"type\":\"int\"},\"copies\":{\"value\":\"\",\"type\":\"int\"},\"attachments\":{\"value\":\"\",\"type\":\"int\"}}");
        context.setMastrid("{\"doc_mark\":{\"value\":\"\",\"type\":\"int\"},\"secret_level\":{\"value\":\"\",\"type\":\"int\"},\"urgent_level\":{\"value\":\"\",\"type\":\"int\"},\"signing_date\":{\"value\":\"\",\"type\":\"int\"},\"copies\":{\"value\":\"\",\"type\":\"int\"},\"attachments\":{\"value\":\"\",\"type\":\"int\"}}");
        context.setDebugMode(false);
        context.setProcessTemplateId("");
        context.setMatchRequestToken(cacheId.toString());
        context.setIsValidate(true);
        
        
        CPMatchResultVO cpMatchResult = new CPMatchResultVO();
        cpMatchResult.setAllNotSelectNodes(new HashSet<String>());
        cpMatchResult.setAllSelectNodes(new HashSet<String>());
        cpMatchResult.setAllSelectInformNodes(new HashSet<String>());
        cpMatchResult.setPop(false);
        cpMatchResult.setToken("");
        cpMatchResult.setLast("false");
        cpMatchResult.setAlreadyChecked("false");
        
        wapi.transBeforeInvokeWorkFlow(context, cpMatchResult);
        
        return cacheId;
    }
    
    /**
     * 创建一条普通的公文, 节点有期限， 发起人是edoc1，节点
     * @return 
     * @Author      : xuqw
     * @Date        : 2015年10月13日下午3:11:21
     * @throws Exception
     */
    public static EdocSummary createEdocSummaryWithNodeDeadlin(String subject) throws Exception {

        V3xOrgMember edoc1 = orgManager.getMemberByLoginName("edoc1");
        
        int key = 0;
        long nodeId = SeeyonTestUtil.get14Id(key++);
        
        String process = "<ps>"
                + "<p s=\"\" u=\"\" y=\"0\" x=\"0\" t=\"p\" d=\"\" n=\"\" i=\"\">"
                + "<n a=\"1\" b=\"normal\" c=\"false\" l=\"1000\" e=\"0\" f=\"\" g=\"\" h=\"\" y=\"50\" x=\"155\" t=\"6\" d=\"\" n=\""+edoc1.getName()+"\" i=\""+nodeId+"\">"
                + "<a a=\"\" b=\""+edoc1.getOrgAccountId()+"\" c=\"1\" d=\""+edoc1.getName()+"\" e=\"\" f=\""+edoc1.getId()+"\" g=\"user\" h=\"false\" i=\"false\" j=\"false\" k=\"roleadmin\" />"
                + "<s sa=\"0\" a=\"\" b=\"0\" c=\"1\" z=\"\" r=\"\" e=\"\" f=\"\" g=\"0\" na=\"0\" w=\"1\" v=\"0\" sid=\"\" qid=\"\" h=\"-1\" u=\"-1\" tm=\"1\" j=\"single\" k=\"5\" l=\"60\" m=\"false\" s=\"success\" o=\"null\" p=\"null\" q=\"null\" d=\"\" t=\"\" n=\"审批\" i=\"shenpi\" />"
                + "</n>"
                + "<n f=\"\" g=\"\" h=\"\" y=\"50\" x=\"260\" t=\"4\" d=\"\" n=\"end\" i=\"end\">"
                + "<a a=\"\" b=\"\" c=\"1\" d=\"\" e=\"\" f=\"\" g=\"\" h=\"false\" i=\"false\" j=\"false\" k=\"\" />"
                + "<s sa=\"0\" a=\"\" b=\"0\" c=\"1\" z=\"\" r=\"\" e=\"\" f=\"\" g=\"0\" na=\"-1\" w=\"-1\" v=\"-1\" sid=\"\" qid=\"\" h=\"-1\" u=\"-1\" tm=\"1\" j=\"multiple\" k=\"\" l=\"\" m=\"false\" s=\"success\" o=\"\" p=\"\" q=\"\" d=\"\" t=\"\" n=\"审批\" i=\"shenpi\" />"
                + "</n>"
                + "<n f=\"\" g=\"\" h=\"\" y=\"50\" x=\"50\" t=\"8\" d=\"\" n=\"\" i=\"start\">"
                + "<a a=\"\" b=\"\" c=\"1\" d=\"\" e=\"\" f=\"\" g=\"\" h=\"false\" i=\"false\" j=\"false\" k=\"\" />"
                + "<s sa=\"0\" a=\"\" b=\"0\" c=\"1\" z=\"\" r=\"\" e=\"\" f=\"\" g=\"0\" na=\"-1\" w=\"-1\" v=\"-1\" sid=\"\" qid=\"\" h=\"-1\" u=\"-1\" tm=\"1\" j=\"multiple\" k=\"\" l=\"\" m=\"false\" s=\"success\" o=\"\" p=\"\" q=\"\" d=\"\" t=\"\" n=\"\" i=\"shenpi\" />"
                + "</n>"
                + "<l a=\"\" b=\"\" c=\"\" m=\"\" e=\"\" h=\"3\" o=\"0\" j=\""+nodeId+"\" k=\"start\" t=\"11\" d=\"\" n=\"\" i=\""+SeeyonTestUtil.get14Id(key++)+"\" />"
                + "<l a=\"\" b=\"\" c=\"\" m=\"\" e=\"\" h=\"3\" o=\"0\" j=\"end\" k=\""+nodeId+"\" t=\"11\" d=\"\" n=\"\" i=\""+SeeyonTestUtil.get14Id(key++)+"\" />"
                + "</p>"
                + "</ps>";
        
        User user = SeeyonTestUtil.createSessionUser(edoc1);
        AppContext.putThreadContext(GlobalNames.SESSION_CONTEXT_USERINFO_KEY, user);
        
        long casheId = doWfPreSubmit(process, "sendEdoc", edoc1);
        
        process = "&process_xml=" + process + "&workflow_node_condition_input={\"matchRequestToken\":\""+ casheId +"\",\"condition\":[{\"nodeId\":\""+nodeId+"\",\"isDelete\":\"false\"}]}";
        
        return createSimpleEdocSummary(subject, process);
    }
    
    /**
     * 创建公文模板数据
     * @return
     * @throws BusinessException 
     * 
     * @Author      : xuqw
     * @Date        : 2016年5月30日上午9:54:17
     */
    public static CtpTemplate createEdocTemplate(V3xOrgMember creater, EdocSummary summary)
                                                    throws BusinessException{
        
        CtpTemplate template = new CtpTemplate();
        
        Long id = UUIDLong.longUUID();
        
        template.setId(id);
        template.setCategoryId((long) 19);//发文
        template.setModuleType(19);
        template.setType("templete");
        template.setSubject("单元测试_" + id);
        template.setSystem(true);
        template.setMemberId(creater.getId());
        template.setCreateDate(new Date());
        template.setModifyMember(creater.getId());
        template.setModifyDate(new Date());
        
        template.setSummary(XMLCoder.encoder(summary));
        
        template.setWorkflowId(-1l);
        template.setBody(-1l);
        template.setBodyType("10");
        template.setState(0);
        template.setDelete(false);
        template.setOrgAccountId(creater.getOrgAccountId());
        
        templateManager.saveCtpTemplate(template);
        
        return template;
    }
    
    
    /**
     * 设置创建会议数据的默认参数
     * @throws BusinessException 
     * @Author      : xuqw
     * @Date        : 2015年9月28日上午12:09:13
     */
    private static void createDefualtMap() throws BusinessException {

        defaultFieldMap = new HashMap<String, String>();

        SimpleDateFormat df = new SimpleDateFormat("yyyy-MM-dd HH:mm");

        Date nowdate = new Date();
        Date beginDate = new Date(nowdate.getTime() + 30 * 60 * 1000);// 半个小时后开始

        //V3xOrgMember meeting1 = orgManager.getMemberByLoginName("meeting1");
        //V3xOrgMember meeting2 = orgManager.getMemberByLoginName("meeting2");

        //defaultFieldMap.put(FIELD_ISOPENWINDOW, "true");
    }
    
}
